@@ -2,17 +2,17 @@
 #define LIBRSVG_FEATURES_H 1
 
 #define LIBRSVG_MAJOR_VERSION (2)
-#define LIBRSVG_MINOR_VERSION (34)
-#define LIBRSVG_MICRO_VERSION (1)
-#define LIBRSVG_VERSION "2.34.1"
+#define LIBRSVG_MINOR_VERSION (35)
+#define LIBRSVG_MICRO_VERSION (0)
+#define LIBRSVG_VERSION "2.35.0"
 
 #define LIBRSVG_CHECK_VERSION(major,minor,micro) \
   (LIBRSVG_MAJOR_VERSION > (major) || \
    (LIBRSVG_MAJOR_VERSION == (major) && LIBRSVG_MINOR_VERSION > (minor)) || \
    (LIBRSVG_MAJOR_VERSION == (major) && LIBRSVG_MINOR_VERSION == (minor) && LIBRSVG_MICRO_VERSION >= (micro)))
 
-#define LIBRSVG_HAVE_SVGZ  (1)
-#define LIBRSVG_HAVE_CSS   (1)
+#define LIBRSVG_HAVE_SVGZ  (TRUE)
+#define LIBRSVG_HAVE_CSS   (TRUE)
 
 #define LIBRSVG_CHECK_FEATURE(FEATURE) (defined(LIBRSVG_HAVE_##FEATURE) && LIBRSVG_HAVE_##FEATURE)
 
