@@ -26,13 +26,6 @@
 
 #include "config.h"
 
-#ifdef HAVE_SVGZ
-#include <gsf/gsf-input-gzip.h>
-#include <gsf/gsf-input-memory.h>
-#include <gsf/gsf-output-memory.h>
-#include <gsf/gsf-utils.h>
-#endif
-
 #include "rsvg.h"
 #include "rsvg-private.h"
 #include "rsvg-css.h"
@@ -47,14 +40,18 @@
 #include "rsvg-cairo-render.h"
 
 #include <libxml/uri.h>
+#include <libxml/parser.h>
+#include <libxml/parserInternals.h>
+
+#include <gio/gio.h>
 
 #include <math.h>
 #include <string.h>
 #include <stdarg.h>
 
-#include "rsvg-bpath-util.h"
 #include "rsvg-path.h"
 #include "rsvg-paint-server.h"
+#include "rsvg-xml.h"
 
 /*
  * This is configurable at runtime
@@ -124,7 +121,7 @@ rsvg_style_handler_end (RsvgSaxHandler * self, const char *name)
 }
 
 static void
-rsvg_start_style (RsvgHandle * ctx, RsvgPropertyBag * atts)
+rsvg_start_style (RsvgHandle * ctx)
 {
     RsvgSaxHandlerStyle *handler = g_new0 (RsvgSaxHandlerStyle, 1);
 
@@ -144,7 +141,6 @@ rsvg_start_style (RsvgHandle * ctx, RsvgPropertyBag * atts)
 static void
 rsvg_standard_element_start (RsvgHandle * ctx, const char *name, RsvgPropertyBag * atts)
 {
-
     /*replace this stuff with a hash for fast reading! */
     RsvgNode *newnode = NULL;
     if (!strcmp (name, "g"))
@@ -238,11 +234,11 @@ rsvg_standard_element_start (RsvgHandle * ctx, const char *name, RsvgPropertyBag
     else if (!strcmp (name, "feFuncA"))
         newnode = rsvg_new_node_component_transfer_function ('a');
     else if (!strcmp (name, "feDistantLight"))
-        newnode = rsvg_new_filter_primitive_light_source ('d');
+        newnode = rsvg_new_node_light_source ('d');
     else if (!strcmp (name, "feSpotLight"))
-        newnode = rsvg_new_filter_primitive_light_source ('s');
+        newnode = rsvg_new_node_light_source ('s');
     else if (!strcmp (name, "fePointLight"))
-        newnode = rsvg_new_filter_primitive_light_source ('p');
+        newnode = rsvg_new_node_light_source ('p');
     /* hack to make multiImage sort-of work */
     else if (!strcmp (name, "multiImage"))
         newnode = rsvg_new_switch ();
@@ -256,21 +252,22 @@ rsvg_standard_element_start (RsvgHandle * ctx, const char *name, RsvgPropertyBag
         newnode = rsvg_new_tspan ();
     else if (!strcmp (name, "tref"))
         newnode = rsvg_new_tref ();
-	else {
+    else {
 		/* hack for bug 401115. whenever we encounter a node we don't understand, push it into a group. 
 		   this will allow us to handle things like conditionals properly. */
 		newnode = rsvg_new_group ();
 	}
 
     if (newnode) {
-        newnode->type = g_string_new (name);
+        g_assert (RSVG_NODE_TYPE (newnode) != RSVG_NODE_TYPE_INVALID);
+        newnode->name = (char *) name; /* libxml will keep this while parsing */
         newnode->parent = ctx->priv->currentnode;
         rsvg_node_set_atts (newnode, ctx, atts);
         rsvg_defs_register_memory (ctx->priv->defs, newnode);
         if (ctx->priv->currentnode) {
             rsvg_node_group_pack (ctx->priv->currentnode, newnode);
             ctx->priv->currentnode = newnode;
-        } else if (!strcmp (name, "svg")) {
+        } else if (RSVG_NODE_TYPE (newnode) == RSVG_NODE_TYPE_SVG) {
             ctx->priv->treebase = newnode;
             ctx->priv->currentnode = newnode;
         }
@@ -327,7 +324,7 @@ rsvg_desc_handler_end (RsvgSaxHandler * self, const char *name)
 }
 
 static void
-rsvg_start_desc (RsvgHandle * ctx, RsvgPropertyBag * atts)
+rsvg_start_desc (RsvgHandle * ctx)
 {
     RsvgSaxHandlerDesc *handler = g_new0 (RsvgSaxHandlerDesc, 1);
 
@@ -393,7 +390,7 @@ rsvg_title_handler_end (RsvgSaxHandler * self, const char *name)
 }
 
 static void
-rsvg_start_title (RsvgHandle * ctx, RsvgPropertyBag * atts)
+rsvg_start_title (RsvgHandle * ctx)
 {
     RsvgSaxHandlerTitle *handler = g_new0 (RsvgSaxHandlerTitle, 1);
 
@@ -473,7 +470,7 @@ rsvg_metadata_handler_end (RsvgSaxHandler * self, const char *name)
 }
 
 static void
-rsvg_start_metadata (RsvgHandle * ctx, RsvgPropertyBag * atts)
+rsvg_start_metadata (RsvgHandle * ctx)
 {
     RsvgSaxHandlerMetadata *handler = g_new0 (RsvgSaxHandlerMetadata, 1);
 
@@ -599,11 +596,10 @@ rsvg_start_xinclude (RsvgHandle * ctx, RsvgPropertyBag * atts)
                 /* xml */
                 xmlDocPtr xml_doc;
                 xmlParserCtxtPtr xml_parser;
-                int result;
 
                 xml_parser = xmlCreatePushParserCtxt (&rsvgSAXHandlerStruct, ctx, NULL, 0, NULL);
-                result = xmlParseChunk (xml_parser, (char *) data->data, data->len, 0);
-                result = xmlParseChunk (xml_parser, "", 0, TRUE);
+                (void) xmlParseChunk (xml_parser, (char *) data->data, data->len, 0);
+                (void) xmlParseChunk (xml_parser, "", 0, TRUE);
 
                 xml_doc = xml_parser->myDoc;
                 xmlFreeParserCtxt (xml_parser);
@@ -650,13 +646,13 @@ rsvg_start_element (void *data, const xmlChar * name, const xmlChar ** atts)
                 name = (const xmlChar *) (tempname + 1);
 
         if (!strcmp ((const char *) name, "style"))
-            rsvg_start_style (ctx, bag);
+            rsvg_start_style (ctx);
         else if (!strcmp ((const char *) name, "title"))
-            rsvg_start_title (ctx, bag);
+            rsvg_start_title (ctx);
         else if (!strcmp ((const char *) name, "desc"))
-            rsvg_start_desc (ctx, bag);
+            rsvg_start_desc (ctx);
         else if (!strcmp ((const char *) name, "metadata"))
-            rsvg_start_metadata (ctx, bag);
+            rsvg_start_metadata (ctx);
         else if (!strcmp ((const char *) name, "include"))      /* xi:include */
             rsvg_start_xinclude (ctx, bag);
         else
@@ -686,11 +682,12 @@ rsvg_end_element (void *data, const xmlChar * name)
             ctx->priv->handler = NULL;
         }
 
-        if (ctx->priv->currentnode
-            && !strcmp ((const char *) name, ctx->priv->currentnode->type->str))
-            rsvg_pop_def_group (ctx);
+        if (ctx->priv->currentnode &&
+            !strcmp ((const char *) name, ctx->priv->currentnode->name))
+                rsvg_pop_def_group (ctx);
 
-        if (!strcmp ((const char *)name, "style"))
+        /* FIXMEchpe: shouldn't this check that currentnode == treebase or sth like that? */
+        if (ctx->priv->treebase && !strcmp ((const char *)name, "svg"))
             _rsvg_node_svg_apply_atts ((RsvgNodeSvg *)ctx->priv->treebase, ctx);
     }
 }
@@ -703,6 +700,30 @@ _rsvg_node_chars_free (RsvgNode * node)
     _rsvg_node_free (node);
 }
 
+static RsvgNodeChars *
+rsvg_new_node_chars (const char *text,
+                     int len)
+{
+    RsvgNodeChars *self;
+
+    self = g_new (RsvgNodeChars, 1);
+    _rsvg_node_init (&self->super, RSVG_NODE_TYPE_CHARS);
+
+    if (!g_utf8_validate (text, len, NULL)) {
+        char *utf8;
+        utf8 = rsvg_make_valid_utf8 (text, len);
+        self->contents = g_string_new (utf8);
+        g_free (utf8);
+    } else {
+        self->contents = g_string_new_len (text, len);
+    }
+
+    self->super.free = _rsvg_node_chars_free;
+    self->super.state->cond_true = FALSE;
+
+    return self;
+}
+
 static void
 rsvg_characters_impl (RsvgHandle * ctx, const xmlChar * ch, int len)
 {
@@ -712,8 +733,9 @@ rsvg_characters_impl (RsvgHandle * ctx, const xmlChar * ch, int len)
         return;
 
     if (ctx->priv->currentnode) {
-        if (!strcmp ("tspan", ctx->priv->currentnode->type->str) ||
-            !strcmp ("text", ctx->priv->currentnode->type->str)) {
+        RsvgNodeType type = RSVG_NODE_TYPE (ctx->priv->currentnode);
+        if (type == RSVG_NODE_TYPE_TSPAN ||
+            type == RSVG_NODE_TYPE_TEXT) {
             guint i;
 
             /* find the last CHARS node in the text or tspan node, so that we
@@ -721,9 +743,12 @@ rsvg_characters_impl (RsvgHandle * ctx, const xmlChar * ch, int len)
             self = NULL;
             for (i = 0; i < ctx->priv->currentnode->children->len; i++) {
                 RsvgNode *node = g_ptr_array_index (ctx->priv->currentnode->children, i);
-                if (!strcmp (node->type->str, "RSVG_NODE_CHARS")) {
+                if (RSVG_NODE_TYPE (node) == RSVG_NODE_TYPE_CHARS) {
                     self = (RsvgNodeChars*)node;
                 }
+                else if (RSVG_NODE_TYPE (node) == RSVG_NODE_TYPE_TSPAN) {
+                    self = NULL;
+                }
             }
 
             if (self != NULL) {
@@ -741,21 +766,7 @@ rsvg_characters_impl (RsvgHandle * ctx, const xmlChar * ch, int len)
         }
     }
 
-    self = g_new (RsvgNodeChars, 1);
-    _rsvg_node_init (&self->super);
-
-    if (!g_utf8_validate ((char *) ch, len, NULL)) {
-        char *utf8;
-        utf8 = rsvg_make_valid_utf8 ((char *) ch, len);
-        self->contents = g_string_new (utf8);
-        g_free (utf8);
-    } else {
-        self->contents = g_string_new_len ((char *) ch, len);
-    }
-
-    self->super.type = g_string_new ("RSVG_NODE_CHARS");
-    self->super.free = _rsvg_node_chars_free;
-    self->super.state->cond_true = FALSE;
+    self = rsvg_new_node_chars ((char *) ch, len);
 
     rsvg_defs_register_memory (ctx->priv->defs, (RsvgNode *) self);
     if (ctx->priv->currentnode)
@@ -775,14 +786,6 @@ rsvg_characters (void *data, const xmlChar * ch, int len)
     rsvg_characters_impl (ctx, ch, len);
 }
 
-#if LIBXML_VERSION >= 20621
-#define RSVG_ENABLE_ENTITIES
-#elif defined(__GNUC__)
-#warning "libxml version less than 2.6.22. XML entities won't work"
-#endif
-
-#ifdef RSVG_ENABLE_ENTITIES
-
 static xmlEntityPtr
 rsvg_get_entity (void *data, const xmlChar * name)
 {
@@ -801,7 +804,6 @@ rsvg_entity_decl (void *data, const xmlChar * name, int type,
     RsvgHandle *ctx = (RsvgHandle *) data;
     GHashTable *entities = ctx->priv->entities;
     xmlEntityPtr entity;
-#if LIBXML_VERSION >= 20700
     xmlChar *resolvedSystemId = NULL, *resolvedPublicId = NULL;
 
     if (systemId)
@@ -826,50 +828,10 @@ rsvg_entity_decl (void *data, const xmlChar * name, int type,
       
     entity = xmlNewEntity(NULL, name, type, resolvedPublicId, resolvedSystemId, content);
 
-    free(resolvedPublicId);
-    free(resolvedSystemId);
+    xmlFree(resolvedPublicId);
+    xmlFree(resolvedSystemId);
 
     g_hash_table_insert (entities, g_strdup ((const char*) name), entity);
-#else
-    xmlChar *dupname;
-
-    entity = xmlMalloc (sizeof (xmlEntity));
-    memset (entity, 0, sizeof (xmlEntity));
-    entity->type = XML_ENTITY_DECL;
-    dupname = (xmlChar *) xmlMemStrdup ((const char *) name);
-    entity->name = dupname;
-
-    entity->etype = type;
-    if (content) {
-        entity->content = (xmlChar *) xmlMemStrdup ((const char *) content);
-        entity->length = strlen ((const char *) content);
-    } else if (systemId || publicId) {
-        GByteArray *data = NULL;
-
-        if (systemId)
-            data =
-                _rsvg_acquire_xlink_href_resource ((const char *) systemId,
-                                                   rsvg_handle_get_base_uri (ctx), NULL);
-        else if (publicId)
-            data =
-                _rsvg_acquire_xlink_href_resource ((const char *) publicId,
-                                                   rsvg_handle_get_base_uri (ctx), NULL);
-
-        if (data) {
-            entity->SystemID = (xmlChar *) xmlMemStrdup ((const char *) systemId);
-            entity->ExternalID = (xmlChar *) xmlMemStrdup ((const char *) publicId);
-            entity->content = (xmlChar *) xmlMemStrdup ((const char *) data->data);
-            entity->length = data->len;
-
-            /* fool libxml2 into supporting SYSTEM and PUBLIC entities */
-            entity->etype = XML_INTERNAL_GENERAL_ENTITY;
-
-            g_byte_array_free (data, TRUE);
-        }
-    }
-
-    g_hash_table_insert (entities, dupname, entity);
-#endif
 }
 
 static void
@@ -892,8 +854,6 @@ rsvg_get_parameter_entity (void *data, const xmlChar * name)
     return entity;
 }
 
-#endif
-
 static void
 rsvg_error_cb (void *data, const char *msg, ...)
 {
@@ -917,17 +877,17 @@ rsvg_processing_instruction (void *ctx, const xmlChar * target, const xmlChar *
         char **xml_atts;
 
         xml_atts = rsvg_css_parse_xml_attribute_string ((const char *) data);
-        atts = rsvg_property_bag_new ((const char **) xml_atts);
 
-        if (atts) {
+        if (xml_atts) {
             const char *value;
 
+            atts = rsvg_property_bag_new ((const char **) xml_atts);
             value = rsvg_property_bag_lookup (atts, "alternate");
-            if (!value || (strcmp (value, "no") != 0)) {
+            if (!value || !value[0] || (strcmp (value, "no") != 0)) {
                 value = rsvg_property_bag_lookup (atts, "type");
                 if (value && strcmp (value, "text/css") == 0) {
                     value = rsvg_property_bag_lookup (atts, "href");
-                    if (value) {
+                    if (value && value[0]) {
                         GByteArray *style;
 
                         style =
@@ -942,8 +902,8 @@ rsvg_processing_instruction (void *ctx, const xmlChar * target, const xmlChar *
                 }
             }
 
-            g_strfreev (xml_atts);
             rsvg_property_bag_free (atts);
+            g_strfreev (xml_atts);
         }
     }
 }
@@ -956,12 +916,10 @@ rsvg_SAX_handler_struct_init (void)
 
         memset (&rsvgSAXHandlerStruct, 0, sizeof (rsvgSAXHandlerStruct));
 
-#ifdef RSVG_ENABLE_ENTITIES
         rsvgSAXHandlerStruct.getEntity = rsvg_get_entity;
         rsvgSAXHandlerStruct.entityDecl = rsvg_entity_decl;
         rsvgSAXHandlerStruct.unparsedEntityDecl = rsvg_unparsed_entity_decl;
         rsvgSAXHandlerStruct.getParameterEntity = rsvg_get_parameter_entity;
-#endif
         rsvgSAXHandlerStruct.characters = rsvg_characters;
         rsvgSAXHandlerStruct.error = rsvg_error_cb;
         rsvgSAXHandlerStruct.cdataBlock = rsvg_characters;
@@ -1057,6 +1015,39 @@ rsvg_handle_set_base_uri (RsvgHandle * handle, const char *base_uri)
     }
 }
 
+/**
+ * rsvg_handle_set_base_gfile:
+ * @handle: a #RsvgHandle
+ * @file: a #GFile
+ *
+ * Set the base URI for @handle from @file.
+ * Note: This function may only be called before rsvg_handle_write()
+ * or rsvg_handle_read_stream_sync() has been called.
+ *
+ * Since: 2.32
+ */
+void
+rsvg_handle_set_base_gfile (RsvgHandle *handle,
+                            GFile      *base_file)
+{
+    RsvgHandlePrivate *priv;
+
+    g_return_if_fail (RSVG_IS_HANDLE (handle));
+    g_return_if_fail (G_IS_FILE (base_file));
+
+    priv = handle->priv;
+
+    g_object_ref (base_file);
+    if (priv->base_gfile)
+        g_object_unref (priv->base_gfile);
+    priv->base_gfile = base_file;
+    
+    g_free (priv->base_uri);
+    priv->base_uri = g_file_get_uri (base_file);
+
+    rsvg_defs_set_base_uri (priv->defs, priv->base_uri);
+}
+
 /**
  * rsvg_handle_get_base_uri:
  * @handle: A #RsvgHandle
@@ -1064,9 +1055,9 @@ rsvg_handle_set_base_uri (RsvgHandle * handle, const char *base_uri)
  * Gets the base uri for this #RsvgHandle.
  *
  * Returns: the base uri, possibly null
- * Since: 2.9 (really present in 2.8 as well)
+ * Since: 2.8
  */
-G_CONST_RETURN char *
+const char *
 rsvg_handle_get_base_uri (RsvgHandle * handle)
 {
     g_return_val_if_fail (handle, NULL);
@@ -1157,13 +1148,15 @@ rsvg_handle_close_impl (RsvgHandle * handle, GError ** error)
         xmlDoc = handle->priv->ctxt->myDoc;
 
         result = xmlParseChunk (handle->priv->ctxt, "", 0, TRUE);
-        xmlFreeParserCtxt (handle->priv->ctxt);
-        xmlFreeDoc (xmlDoc);
-
         if (result != 0) {
             rsvg_set_error (error, handle->priv->ctxt);
+            xmlFreeParserCtxt (handle->priv->ctxt);
+            xmlFreeDoc (xmlDoc);
             return FALSE;
         }
+
+        xmlFreeParserCtxt (handle->priv->ctxt);
+        xmlFreeDoc (xmlDoc);
     }
 
     rsvg_defs_resolve_all (handle->priv->defs);
@@ -1178,20 +1171,12 @@ rsvg_handle_close_impl (RsvgHandle * handle, GError ** error)
     return TRUE;
 }
 
-static void
-rsvg_state_free_func (gpointer data, gpointer user_data)
-{
-    rsvg_state_finalize ((RsvgState *) data);
-    g_slice_free (RsvgState, data);
-}
-
 void
 rsvg_drawing_ctx_free (RsvgDrawingCtx * handle)
 {
     rsvg_render_free (handle->render);
 
-    g_slist_foreach (handle->state, rsvg_state_free_func, (gpointer) handle);
-    g_slist_free (handle->state);
+    rsvg_state_free_all (handle->state);
 
 	/* the drawsub stack's nodes are owned by the ->defs */
 	g_slist_free (handle->drawsub_stack);
@@ -1218,7 +1203,7 @@ rsvg_drawing_ctx_free (RsvgDrawingCtx * handle)
  *
  * Since: 2.9
  */
-G_CONST_RETURN char *
+const char *
 rsvg_handle_get_metadata (RsvgHandle * handle)
 {
     g_return_val_if_fail (handle, NULL);
@@ -1240,7 +1225,7 @@ rsvg_handle_get_metadata (RsvgHandle * handle)
  *
  * Since: 2.4
  */
-G_CONST_RETURN char *
+const char *
 rsvg_handle_get_title (RsvgHandle * handle)
 {
     g_return_val_if_fail (handle, NULL);
@@ -1262,7 +1247,7 @@ rsvg_handle_get_title (RsvgHandle * handle)
  *
  * Since: 2.4
  */
-G_CONST_RETURN char *
+const char *
 rsvg_handle_get_desc (RsvgHandle * handle)
 {
     g_return_val_if_fail (handle, NULL);
@@ -1276,7 +1261,7 @@ rsvg_handle_get_desc (RsvgHandle * handle)
 /**
  * rsvg_handle_get_dimensions
  * @handle: A #RsvgHandle
- * @dimension_data: A place to store the SVG's size
+ * @dimension_data: (out): A place to store the SVG's size
  *
  * Get the SVG's size. Do not call from within the size_func callback, because an infinite loop will occur.
  *
@@ -1317,7 +1302,6 @@ rsvg_handle_get_dimensions_sub (RsvgHandle * handle, RsvgDimensionData * dimensi
     cairo_t *cr;
     cairo_surface_t *target;
     RsvgDrawingCtx *draw;
-    RsvgCairoRender *render;
     RsvgNodeSvg *root = NULL;
     RsvgNode *sself = NULL;
     RsvgBbox bbox;
@@ -1346,8 +1330,8 @@ rsvg_handle_get_dimensions_sub (RsvgHandle * handle, RsvgDimensionData * dimensi
     if (!root)
         return FALSE;
 
-    bbox.x = bbox.y = 0;
-    bbox.w = bbox.h = 1;
+    bbox.rect.x = bbox.rect.y = 0;
+    bbox.rect.width = bbox.rect.height = 1;
 
     if (!id && (root->w.factor == 'p' || root->h.factor == 'p')
             && !root->vbox.active)
@@ -1378,12 +1362,7 @@ rsvg_handle_get_dimensions_sub (RsvgHandle * handle, RsvgDimensionData * dimensi
         cairo_save (cr);
 
         rsvg_node_draw ((RsvgNode *) handle->priv->treebase, draw, 0);
-        render = (RsvgCairoRender *) draw->render;
-
-        bbox.x = render->bbox.x;
-        bbox.y = render->bbox.y;
-        bbox.w = render->bbox.w;
-        bbox.h = render->bbox.h;
+        bbox = RSVG_CAIRO_RENDER (draw->render)->bbox;
 
         cairo_restore (cr);
         rsvg_state_pop (draw);
@@ -1391,16 +1370,16 @@ rsvg_handle_get_dimensions_sub (RsvgHandle * handle, RsvgDimensionData * dimensi
         cairo_destroy (cr);
         cairo_surface_destroy (target);
 
-        dimension_data->width = bbox.w;
-        dimension_data->height = bbox.h;
+        dimension_data->width = bbox.rect.width;
+        dimension_data->height = bbox.rect.height;
     } else {
-        bbox.w = root->vbox.w;
-        bbox.h = root->vbox.h;
+        bbox.rect.width = root->vbox.rect.width;
+        bbox.rect.height = root->vbox.rect.height;
 
         dimension_data->width = (int) (_rsvg_css_hand_normalize_length (&root->w, handle->priv->dpi_x,
-                                       bbox.w + bbox.x * 2, 12) + 0.5);
+                                       bbox.rect.width + bbox.rect.x * 2, 12) + 0.5);
         dimension_data->height = (int) (_rsvg_css_hand_normalize_length (&root->h, handle->priv->dpi_y,
-                                         bbox.h + bbox.y * 2,
+                                         bbox.rect.height + bbox.rect.y * 2,
                                          12) + 0.5);
     }
     
@@ -1481,19 +1460,16 @@ rsvg_handle_get_position_sub (RsvgHandle * handle, RsvgPositionData * position_d
     cairo_save (cr);
 
     rsvg_node_draw ((RsvgNode *) handle->priv->treebase, draw, 0);
-    bbox.x = ((RsvgCairoRender *) draw->render)->bbox.x;
-    bbox.y = ((RsvgCairoRender *) draw->render)->bbox.y;
-    bbox.w = ((RsvgCairoRender *) draw->render)->bbox.w;
-    bbox.h = ((RsvgCairoRender *) draw->render)->bbox.h;
+    bbox = RSVG_CAIRO_RENDER (draw->render)->bbox;
 
     cairo_restore (cr);
     rsvg_state_pop (draw);
     rsvg_drawing_ctx_free (draw);
 
-    position_data->x = bbox.x;
-    position_data->y = bbox.y;
-    dimension_data.width = bbox.w;
-    dimension_data.height = bbox.h;
+    position_data->x = bbox.rect.x;
+    position_data->y = bbox.rect.y;
+    dimension_data.width = bbox.rect.width;
+    dimension_data.height = bbox.rect.height;
 
     dimension_data.em = dimension_data.width;
     dimension_data.ex = dimension_data.height;
@@ -1654,45 +1630,43 @@ rsvg_handle_set_size_callback (RsvgHandle * handle,
 
 /**
  * rsvg_handle_write:
- * @handle: An #RsvgHandle
- * @buf: Pointer to svg data
+ * @handle: an #RsvgHandle
+ * @buf: (array length=count) (element-type uint8): pointer to svg data
  * @count: length of the @buf buffer in bytes
- * @error: return location for errors
+ * @error: (allow-none): a location to store a #GError, or %NULL
  *
  * Loads the next @count bytes of the image.  This will return #TRUE if the data
  * was loaded successful, and #FALSE if an error occurred.  In the latter case,
  * the loader will be closed, and will not accept further writes. If FALSE is
- * returned, @error will be set to an error from the #RSVG_ERROR domain.
+ * returned, @error will be set to an error from the #RsvgError domain. Errors
+ * from #GIOErrorEnum are also possible.
  *
- * Returns: #TRUE if the write was successful, or #FALSE if there was an
- * error.
+ * Returns: %TRUE on success, or %FALSE on error
  **/
 gboolean
 rsvg_handle_write (RsvgHandle * handle, const guchar * buf, gsize count, GError ** error)
 {
-    rsvg_return_val_if_fail (handle, FALSE, error);
-    rsvg_return_val_if_fail (!handle->priv->is_closed, FALSE, error);
+    RsvgHandlePrivate *priv;
 
-    if (handle->priv->first_write) {
-        handle->priv->first_write = FALSE;
+    rsvg_return_val_if_fail (handle, FALSE, error);
+    priv = handle->priv;
+
+    rsvg_return_val_if_fail (!priv->is_closed, FALSE, error);
+
+    if (priv->first_write) {
+        priv->first_write = FALSE;
 
         /* test for GZ marker. todo: store the first 2 bytes in the odd circumstance that someone calls
          * write() in 1 byte increments */
         if ((count >= 2) && (buf[0] == (guchar) 0x1f) && (buf[1] == (guchar) 0x8b)) {
-            handle->priv->is_gzipped = TRUE;
-
-#ifdef HAVE_SVGZ
-            handle->priv->gzipped_data = GSF_OUTPUT (gsf_output_memory_new ());
-#endif
+            priv->data_input_stream = g_memory_input_stream_new ();
         }
     }
 
-    if (handle->priv->is_gzipped) {
-#ifdef HAVE_SVGZ
-        return gsf_output_write (handle->priv->gzipped_data, count, buf);
-#else
-        return FALSE;
-#endif
+    if (priv->data_input_stream) {
+        g_memory_input_stream_add_data ((GMemoryInputStream *) priv->data_input_stream,
+                                        g_memdup (buf, count), count, (GDestroyNotify) g_free);
+        return TRUE;
     }
 
     return rsvg_handle_write_impl (handle, buf, count, error);
@@ -1700,100 +1674,260 @@ rsvg_handle_write (RsvgHandle * handle, const guchar * buf, gsize count, GError
 
 /**
  * rsvg_handle_close:
- * @handle: A #RsvgHandle
- * @error: A #GError
+ * @handle: a #RsvgHandle
+ * @error: (allow-none): a location to store a #GError, or %NULL
  *
  * Closes @handle, to indicate that loading the image is complete.  This will
- * return #TRUE if the loader closed successfully.  Note that @handle isn't
+ * return %TRUE if the loader closed successfully.  Note that @handle isn't
  * freed until @g_object_unref is called.
  *
- * Returns: #TRUE if the loader closed successfully, or #FALSE if there was
- * an error.
+ * Returns: %TRUE on success, or %FALSE on error
  **/
 gboolean
 rsvg_handle_close (RsvgHandle * handle, GError ** error)
 {
+    RsvgHandlePrivate *priv;
+
     rsvg_return_val_if_fail (handle, FALSE, error);
+    priv = handle->priv;
 
-	if (handle->priv->is_closed)
-		return TRUE;
+    if (priv->is_closed)
+          return TRUE;
 
-#if HAVE_SVGZ
-    if (handle->priv->is_gzipped) {
-        GsfInput *gzip;
-        const guchar *bytes;
-        gsize size;
-        gsize remaining;
+    if (priv->data_input_stream) {
+        GConverter *converter;
+        GInputStream *stream;
+        gboolean ret;
 
-        bytes = gsf_output_memory_get_bytes (GSF_OUTPUT_MEMORY (handle->priv->gzipped_data));
-        size = gsf_output_size (handle->priv->gzipped_data);
+        converter = G_CONVERTER (g_zlib_decompressor_new (G_ZLIB_COMPRESSOR_FORMAT_GZIP));
+        stream = g_converter_input_stream_new (priv->data_input_stream, converter);
+        g_object_unref (converter);
+        g_object_unref (priv->data_input_stream);
+        priv->data_input_stream = NULL;
 
-        gzip =
-            GSF_INPUT (gsf_input_gzip_new
-                       (GSF_INPUT (gsf_input_memory_new (bytes, size, FALSE)), error));
-        remaining = gsf_input_remaining (gzip);
-        while ((size = MIN (remaining, 1024)) > 0) {
-            guint8 const *buf;
+        ret = rsvg_handle_read_stream_sync (handle, stream, NULL, error);
+        g_object_unref (stream);
 
-            /* write to parent */
-            buf = gsf_input_read (gzip, size, NULL);
-            if (!buf) {
-                /* an error occured, so bail */
-                g_warning (_("rsvg_gz_handle_close_impl: gsf_input_read returned NULL"));
-                break;
-            }
-
-            rsvg_handle_write_impl (handle, buf, size, error);
-            /* if we didn't manage to lower remaining number of bytes,
-             * something is wrong, and we should avoid an endless loop */
-            if (remaining == ((gsize) gsf_input_remaining (gzip))) {
-                g_warning (_
-                           ("rsvg_gz_handle_close_impl: write_impl didn't lower the input_remaining count"));
-                break;
-            }
-            remaining = gsf_input_remaining (gzip);
-        }
-        g_object_unref (gzip);
-
-        /* close parent */
-        gsf_output_close (handle->priv->gzipped_data);
+        return ret;
     }
-#endif
 
     return rsvg_handle_close_impl (handle, error);
 }
 
+/**
+ * rsvg_handle_read_stream_sync:
+ * @handle: a #RsvgHandle
+ * @stream: a #GInputStream
+ * @cancellable: (allow-none): a #GCancellable, or %NULL
+ * @error: (allow-none): a location to store a #GError, or %NULL
+ *
+ * Reads @stream and writes the data from it to @handle.
+ *
+ * If @cancellable is not %NULL, then the operation can be cancelled by
+ * triggering the cancellable object from another thread. If the
+ * operation was cancelled, the error G_IO_ERROR_CANCELLED will be
+ * returned.
+ *
+ * Returns: %TRUE if reading @stream succeeded, or %FALSE otherwise
+ *   with @error filled in
+ *
+ * Since: 2.32
+ */
+gboolean
+rsvg_handle_read_stream_sync (RsvgHandle   *handle,
+                              GInputStream *stream,
+                              GCancellable *cancellable,
+                              GError      **error)
+{
+    RsvgHandlePrivate *priv;
+    xmlParserInputBufferPtr buffer;
+    xmlParserInputPtr input;
+    int result;
+    xmlDocPtr doc;
+    GError *err = NULL;
+
+    g_return_val_if_fail (RSVG_IS_HANDLE (handle), FALSE);
+    g_return_val_if_fail (G_IS_INPUT_STREAM (stream), FALSE);
+    g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), FALSE);
+    g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+    priv = handle->priv;
+
+    priv->error = &err;
+    if (priv->ctxt == NULL) {
+        priv->ctxt = xmlCreatePushParserCtxt (&rsvgSAXHandlerStruct, handle, NULL, 0,
+                                              rsvg_handle_get_base_uri (handle));
+
+        /* if false, external entities work, but internal ones don't. if true, internal entities
+           work, but external ones don't. favor internal entities, in order to not cause a
+           regression */
+        /* FIXMEchpe: FIX THIS! */
+        priv->ctxt->replaceEntities = TRUE;
+    }
+
+    buffer = _rsvg_xml_input_buffer_new_from_stream (stream, cancellable, XML_CHAR_ENCODING_NONE, &err);
+    input = xmlNewIOInputStream (priv->ctxt, buffer, XML_CHAR_ENCODING_NONE);
+
+    if (xmlPushInput (priv->ctxt, input) < 0) {
+        rsvg_set_error (error, priv->ctxt);
+        xmlFreeInputStream (input);
+        return FALSE;
+    }
+
+    result = xmlParseDocument (priv->ctxt);
+    if (result != 0) {
+        if (err)
+            g_propagate_error (error, err);
+        else
+            rsvg_set_error (error, handle->priv->ctxt);
+
+        return FALSE;
+    }
+
+    priv->error = NULL;
+
+    if (err != NULL) {
+        g_propagate_error (error, err);
+        return FALSE;
+    }
+
+    doc = priv->ctxt->myDoc;
+    xmlFreeParserCtxt (priv->ctxt);
+    priv->ctxt = NULL;
+
+    xmlFreeDoc (doc);
+
+    rsvg_defs_resolve_all (priv->defs);
+    priv->finished = TRUE;
+
+    return TRUE;
+}
+
+/**
+ * rsvg_handle_new_from_gfile_sync:
+ * @file: a #GFile
+ * @flags: flags from #RsvgHandleFlags
+ * @cancellable: (allow-none): a #GCancellable, or %NULL
+ * @error: (allow-none): a location to store a #GError, or %NULL
+ *
+ * Creates a new #RsvgHandle for @file.
+ *
+ * If @cancellable is not %NULL, then the operation can be cancelled by
+ * triggering the cancellable object from another thread. If the
+ * operation was cancelled, the error G_IO_ERROR_CANCELLED will be
+ * returned.
+ *
+ * Returns: a new #RsvgHandle on success, or %NULL with @error filled in
+ *
+ * Since: 2.32
+ */
+RsvgHandle *
+rsvg_handle_new_from_gfile_sync (GFile          *file,
+                                 RsvgHandleFlags flags,
+                                 GCancellable   *cancellable,
+                                 GError        **error)
+{
+    RsvgHandle *handle;
+    GFileInputStream *stream;
+
+    g_return_val_if_fail (G_IS_FILE (file), NULL);
+    g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+    g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+    stream = g_file_read (file, cancellable, error);
+    if (stream == NULL)
+        return NULL;
+
+    handle = rsvg_handle_new_from_stream_sync (G_INPUT_STREAM (stream), file,
+                                               flags, cancellable, error);
+    g_object_unref (stream);
+
+    return handle;
+}
+
+/**
+ * rsvg_handle_new_from_stream_sync:
+ * @stream: a #GInputStream
+ * @base_file: (allow-none): a #GFile, or %NULL
+ * @flags: flags from #RsvgHandleFlags
+ * @cancellable: (allow-none): a #GCancellable, or %NULL
+ * @error: (allow-none): a location to store a #GError, or %NULL
+ *
+ * Creates a new #RsvgHandle for @stream.
+ *
+ * If @cancellable is not %NULL, then the operation can be cancelled by
+ * triggering the cancellable object from another thread. If the
+ * operation was cancelled, the error G_IO_ERROR_CANCELLED will be
+ * returned.
+ *
+ * Returns: a new #RsvgHandle on success, or %NULL with @error filled in
+ *
+ * Since: 2.32
+ */
+RsvgHandle *
+rsvg_handle_new_from_stream_sync (GInputStream   *stream,
+                                  GFile          *base_file,
+                                  RsvgHandleFlags flags,
+                                  GCancellable    *cancellable,
+                                  GError         **error)
+{
+    RsvgHandle *handle;
+
+    g_return_val_if_fail (G_IS_INPUT_STREAM (stream), NULL);
+    g_return_val_if_fail (base_file == NULL || G_IS_FILE (base_file), NULL);
+    g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+    g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+    handle = rsvg_handle_new_with_flags (flags);
+
+    if (base_file)
+        rsvg_handle_set_base_gfile (handle, base_file);
+
+    if (!rsvg_handle_read_stream_sync (handle, stream, cancellable, error)) {
+        g_object_unref (handle);
+        return NULL;
+    }
+
+    return handle;
+}
+
 /**
  * rsvg_init:
  *
  * Initializes librsvg
  * Since: 2.9
+ * Deprecated: 2.36: Use g_type_init()
  **/
 void
 rsvg_init (void)
 {
     g_type_init ();
-
-#ifdef HAVE_SVGZ
-    gsf_init ();
-#endif
-
-    xmlInitParser ();
 }
 
 /**
  * rsvg_term:
  *
- * De-initializes librsvg
+ * This function does nothing.
+ *
  * Since: 2.9
+ * Deprecated: 2.36
  **/
 void
 rsvg_term (void)
 {
-#ifdef HAVE_SVGZ
-    gsf_shutdown ();
-#endif
+}
 
+/**
+ * rsvg_cleanup:
+ *
+ * This function should not be called from normal programs.
+ * See xmlCleanupParser() for more information.
+ *
+ * Since: 2.36
+ **/
+void
+rsvg_cleanup (void)
+{
     xmlCleanupParser ();
 }
 
@@ -1816,18 +1950,10 @@ rsvg_push_discrete_layer (RsvgDrawingCtx * ctx)
 }
 
 void
-rsvg_render_path (RsvgDrawingCtx * ctx, const char *d)
+rsvg_render_path (RsvgDrawingCtx * ctx, const cairo_path_t *path)
 {
-    /* todo: store and use the bpath higher up */
-    RsvgBpathDef *bpath_def;
-
-    bpath_def = rsvg_parse_path (d);
-    rsvg_bpath_def_art_finish (bpath_def);
-
-    ctx->render->render_path (ctx, bpath_def);
-    rsvg_render_markers (bpath_def, ctx);
-
-    rsvg_bpath_def_free (bpath_def);
+    ctx->render->render_path (ctx, path);
+    rsvg_render_markers (ctx, path);
 }
 
 void
@@ -1855,18 +1981,16 @@ rsvg_render_free (RsvgRender * render)
 }
 
 void
-rsvg_bbox_init (RsvgBbox * self, double *affine)
+rsvg_bbox_init (RsvgBbox * self, cairo_matrix_t *affine)
 {
-    int i;
     self->virgin = 1;
-    for (i = 0; i < 6; i++)
-        self->affine[i] = affine[i];
+    self->affine = *affine;
 }
 
 void
 rsvg_bbox_insert (RsvgBbox * dst, RsvgBbox * src)
 {
-    double affine[6];
+    cairo_matrix_t affine;
     double xmin, ymin;
     double xmax, ymax;
     int i;
@@ -1875,21 +1999,24 @@ rsvg_bbox_insert (RsvgBbox * dst, RsvgBbox * src)
         return;
 
     if (!dst->virgin) {
-        xmin = dst->x, ymin = dst->y;
-        xmax = dst->x + dst->w, ymax = dst->y + dst->h;
+        xmin = dst->rect.x, ymin = dst->rect.y;
+        xmax = dst->rect.x + dst->rect.width, ymax = dst->rect.y + dst->rect.height;
     } else {
         xmin = ymin = xmax = ymax = 0;
     }
 
-    _rsvg_affine_invert (affine, dst->affine);
-    _rsvg_affine_multiply (affine, src->affine, affine);
+    affine = dst->affine;
+    if (cairo_matrix_invert (&affine) != CAIRO_STATUS_SUCCESS)
+      return; //FIXMEchpe correct??
+
+    cairo_matrix_multiply (&affine, &src->affine, &affine);
 
     for (i = 0; i < 4; i++) {
         double rx, ry, x, y;
-        rx = src->x + src->w * (double) (i % 2);
-        ry = src->y + src->h * (double) (i / 2);
-        x = affine[0] * rx + affine[2] * ry + affine[4];
-        y = affine[1] * rx + affine[3] * ry + affine[5];
+        rx = src->rect.x + src->rect.width * (double) (i % 2);
+        ry = src->rect.y + src->rect.height * (double) (i / 2);
+        x = affine.xx * rx + affine.xy * ry + affine.x0;
+        y = affine.yx * rx + affine.yy * ry + affine.y0;
         if (dst->virgin) {
             xmin = xmax = x;
             ymin = ymax = y;
@@ -1905,16 +2032,16 @@ rsvg_bbox_insert (RsvgBbox * dst, RsvgBbox * src)
                 ymax = y;
         }
     }
-    dst->x = xmin;
-    dst->y = ymin;
-    dst->w = xmax - xmin;
-    dst->h = ymax - ymin;
+    dst->rect.x = xmin;
+    dst->rect.y = ymin;
+    dst->rect.width = xmax - xmin;
+    dst->rect.height = ymax - ymin;
 }
 
 void
 rsvg_bbox_clip (RsvgBbox * dst, RsvgBbox * src)
 {
-    double affine[6];
+    cairo_matrix_t affine;
 	double xmin, ymin;
 	double xmax, ymax;
     int i;
@@ -1923,21 +2050,24 @@ rsvg_bbox_clip (RsvgBbox * dst, RsvgBbox * src)
         return;
 
 	if (!dst->virgin) {
-        xmin = dst->x + dst->w, ymin = dst->y + dst->h;
-        xmax = dst->x, ymax = dst->y;
+        xmin = dst->rect.x + dst->rect.width, ymin = dst->rect.y + dst->rect.height;
+        xmax = dst->rect.x, ymax = dst->rect.y;
     } else {
         xmin = ymin = xmax = ymax = 0;
     }
 
-    _rsvg_affine_invert (affine, dst->affine);
-    _rsvg_affine_multiply (affine, src->affine, affine);
+    affine = dst->affine;
+    if (cairo_matrix_invert (&affine) != CAIRO_STATUS_SUCCESS)
+      return;
+
+    cairo_matrix_multiply (&affine, &src->affine, &affine);
 
     for (i = 0; i < 4; i++) {
         double rx, ry, x, y;
-        rx = src->x + src->w * (double) (i % 2);
-        ry = src->y + src->h * (double) (i / 2);
-        x = affine[0] * rx + affine[2] * ry + affine[4];
-        y = affine[1] * rx + affine[3] * ry + affine[5];
+        rx = src->rect.x + src->rect.width * (double) (i % 2);
+        ry = src->rect.y + src->rect.height * (double) (i / 2);
+        x = affine.xx * rx + affine.xy * ry + affine.x0;
+        y = affine.yx * rx + affine.yy * ry + affine.y0;
         if (dst->virgin) {
             xmin = xmax = x;
             ymin = ymax = y;
@@ -1954,19 +2084,19 @@ rsvg_bbox_clip (RsvgBbox * dst, RsvgBbox * src)
         }
     }
 
-    if (xmin < dst->x)
-        xmin = dst->x;
-    if (ymin < dst->y)
-        ymin = dst->y;
-    if (xmax > dst->x + dst->w)
-        xmax = dst->x + dst->w;
-    if (ymax > dst->y + dst->h)
-        ymax = dst->y + dst->h;
+    if (xmin < dst->rect.x)
+        xmin = dst->rect.x;
+    if (ymin < dst->rect.y)
+        ymin = dst->rect.y;
+    if (xmax > dst->rect.x + dst->rect.width)
+        xmax = dst->rect.x + dst->rect.width;
+    if (ymax > dst->rect.y + dst->rect.height)
+        ymax = dst->rect.y + dst->rect.height;
 
-    dst->x = xmin;
-    dst->w = xmax - xmin;
-    dst->y = ymin;
-    dst->h = ymax - ymin;
+    dst->rect.x = xmin;
+    dst->rect.width = xmax - xmin;
+    dst->rect.y = ymin;
+    dst->rect.height = ymax - ymin;
 }
 
 void
@@ -1975,8 +2105,8 @@ _rsvg_push_view_box (RsvgDrawingCtx * ctx, double w, double h)
     RsvgViewBox *vb = g_new (RsvgViewBox, 1);
     *vb = ctx->vb;
     ctx->vb_stack = g_slist_prepend (ctx->vb_stack, vb);
-    ctx->vb.w = w;
-    ctx->vb.h = h;
+    ctx->vb.rect.width = w;
+    ctx->vb.rect.height = h;
 }
 
 void
