@@ -31,7 +31,6 @@
 #include "rsvg-cairo-render.h"
 #include "rsvg-cairo-clip.h"
 #include "rsvg-styles.h"
-#include "rsvg-bpath-util.h"
 #include "rsvg-path.h"
 #include "rsvg-filter.h"
 #include "rsvg-structure.h"
@@ -44,92 +43,6 @@
 
 static const cairo_user_data_key_t surface_pixel_data_key;
 
-static void
-_rsvg_cairo_set_shape_antialias (cairo_t * cr, ShapeRenderingProperty aa)
-{
-    if (SHAPE_RENDERING_AUTO == aa)
-        cairo_set_antialias (cr, CAIRO_ANTIALIAS_DEFAULT);
-    else if (SHAPE_RENDERING_OPTIMIZE_SPEED == aa)
-        cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);
-    else if (SHAPE_RENDERING_CRISP_EDGES == aa)
-        cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);
-    else if (SHAPE_RENDERING_GEOMETRIC_PRECISION == aa)
-        cairo_set_antialias (cr, CAIRO_ANTIALIAS_DEFAULT);
-}
-
-static void
-_rsvg_cairo_set_text_antialias (cairo_t * cr, TextRenderingProperty aa)
-{
-    if (TEXT_RENDERING_AUTO == aa)
-        cairo_set_antialias (cr, CAIRO_ANTIALIAS_DEFAULT);
-    else if (TEXT_RENDERING_OPTIMIZE_SPEED == aa)
-        cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);
-    else if (TEXT_RENDERING_OPTIMIZE_LEGIBILITY == aa)
-        cairo_set_antialias (cr, CAIRO_ANTIALIAS_DEFAULT);
-    else if (TEXT_RENDERING_GEOMETRIC_PRECISION == aa)
-        cairo_set_antialias (cr, CAIRO_ANTIALIAS_DEFAULT);
-}
-
-static void
-_rsvg_cairo_set_operator (cairo_t * cr, RsvgCompOpType comp_op)
-{
-    cairo_operator_t op;
-
-    switch (comp_op) {
-    case RSVG_COMP_OP_CLEAR:
-        op = CAIRO_OPERATOR_CLEAR;
-        break;
-    case RSVG_COMP_OP_SRC:
-        op = CAIRO_OPERATOR_SOURCE;
-        break;
-    case RSVG_COMP_OP_DST:
-        op = CAIRO_OPERATOR_DEST;
-        break;
-    case RSVG_COMP_OP_DST_OVER:
-        op = CAIRO_OPERATOR_DEST_OVER;
-        break;
-    case RSVG_COMP_OP_SRC_IN:
-        op = CAIRO_OPERATOR_IN;
-        break;
-    case RSVG_COMP_OP_DST_IN:
-        op = CAIRO_OPERATOR_DEST_IN;
-        break;
-    case RSVG_COMP_OP_SRC_OUT:
-        op = CAIRO_OPERATOR_OUT;
-        break;
-    case RSVG_COMP_OP_DST_OUT:
-        op = CAIRO_OPERATOR_DEST_OUT;
-        break;
-    case RSVG_COMP_OP_SRC_ATOP:
-        op = CAIRO_OPERATOR_ATOP;
-        break;
-    case RSVG_COMP_OP_DST_ATOP:
-        op = CAIRO_OPERATOR_DEST_ATOP;
-        break;
-    case RSVG_COMP_OP_XOR:
-        op = CAIRO_OPERATOR_XOR;
-        break;
-    case RSVG_COMP_OP_PLUS:
-        op = CAIRO_OPERATOR_ADD;
-        break;
-    case RSVG_COMP_OP_MULTIPLY:
-    case RSVG_COMP_OP_SCREEN:
-    case RSVG_COMP_OP_OVERLAY:
-    case RSVG_COMP_OP_DARKEN:
-    case RSVG_COMP_OP_LIGHTEN:
-    case RSVG_COMP_OP_COLOR_DODGE:
-    case RSVG_COMP_OP_COLOR_BURN:
-    case RSVG_COMP_OP_HARD_LIGHT:
-    case RSVG_COMP_OP_SOFT_LIGHT:
-    case RSVG_COMP_OP_DIFFERENCE:
-    case RSVG_COMP_OP_EXCLUSION:
-    default:
-        op = CAIRO_OPERATOR_OVER;
-        break;
-    }
-    cairo_set_operator (cr, op);
-}
-
 static void
 rsvg_pixmap_destroy (gchar * pixels, gpointer data)
 {
@@ -164,7 +77,8 @@ _set_source_rsvg_linear_gradient (RsvgDrawingCtx * ctx,
                                   RsvgLinearGradient * linear,
                                   guint32 current_color_rgb, guint8 opacity, RsvgBbox bbox)
 {
-    cairo_t *cr = ((RsvgCairoRender *) ctx->render)->cr;
+    RsvgCairoRender *render = RSVG_CAIRO_RENDER (ctx->render);
+    cairo_t *cr = render->cr;
     cairo_pattern_t *pattern;
     cairo_matrix_t matrix;
     RsvgLinearGradient statlinear;
@@ -185,21 +99,16 @@ _set_source_rsvg_linear_gradient (RsvgDrawingCtx * ctx,
     if (linear->obj_bbox)
         _rsvg_pop_view_box (ctx);
 
-    cairo_matrix_init (&matrix,
-                       linear->affine[0], linear->affine[1],
-                       linear->affine[2], linear->affine[3], linear->affine[4], linear->affine[5]);
+    matrix = linear->affine;
     if (linear->obj_bbox) {
         cairo_matrix_t bboxmatrix;
-        cairo_matrix_init (&bboxmatrix, bbox.w, 0, 0, bbox.h, bbox.x, bbox.y);
+        cairo_matrix_init (&bboxmatrix, bbox.rect.width, 0, 0, bbox.rect.height,
+                           bbox.rect.x, bbox.rect.y);
         cairo_matrix_multiply (&matrix, &matrix, &bboxmatrix);
     }
     cairo_matrix_invert (&matrix);
     cairo_pattern_set_matrix (pattern, &matrix);
-
-    if (linear->spread == RSVG_GRADIENT_REFLECT)
-        cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REFLECT);
-    else if (linear->spread == RSVG_GRADIENT_REPEAT)
-        cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
+    cairo_pattern_set_extend (pattern, linear->spread);
 
     _pattern_add_rsvg_color_stops (pattern, linear->super.children, current_color_rgb, opacity);
 
@@ -212,7 +121,8 @@ _set_source_rsvg_radial_gradient (RsvgDrawingCtx * ctx,
                                   RsvgRadialGradient * radial,
                                   guint32 current_color_rgb, guint8 opacity, RsvgBbox bbox)
 {
-    cairo_t *cr = ((RsvgCairoRender *) ctx->render)->cr;
+    RsvgCairoRender *render = RSVG_CAIRO_RENDER (ctx->render);
+    cairo_t *cr = render->cr;
     cairo_pattern_t *pattern;
     cairo_matrix_t matrix;
     RsvgRadialGradient statradial;
@@ -234,22 +144,17 @@ _set_source_rsvg_radial_gradient (RsvgDrawingCtx * ctx,
     if (radial->obj_bbox)
         _rsvg_pop_view_box (ctx);
 
-    cairo_matrix_init (&matrix,
-                       radial->affine[0], radial->affine[1],
-                       radial->affine[2], radial->affine[3], radial->affine[4], radial->affine[5]);
+    matrix = radial->affine;
     if (radial->obj_bbox) {
         cairo_matrix_t bboxmatrix;
-        cairo_matrix_init (&bboxmatrix, bbox.w, 0, 0, bbox.h, bbox.x, bbox.y);
+        cairo_matrix_init (&bboxmatrix, bbox.rect.width, 0, 0, bbox.rect.height,
+                           bbox.rect.x, bbox.rect.y);
         cairo_matrix_multiply (&matrix, &matrix, &bboxmatrix);
     }
 
     cairo_matrix_invert (&matrix);
     cairo_pattern_set_matrix (pattern, &matrix);
-
-    if (radial->spread == RSVG_GRADIENT_REFLECT)
-        cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REFLECT);
-    else if (radial->spread == RSVG_GRADIENT_REPEAT)
-        cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
+    cairo_pattern_set_extend (pattern, radial->spread);
 
     _pattern_add_rsvg_color_stops (pattern, radial->super.children, current_color_rgb, opacity);
 
@@ -261,7 +166,8 @@ static void
 _set_source_rsvg_solid_colour (RsvgDrawingCtx * ctx,
                                RsvgSolidColour * colour, guint8 opacity, guint32 current_colour)
 {
-    cairo_t *cr = ((RsvgCairoRender *) ctx->render)->cr;
+    RsvgCairoRender *render = RSVG_CAIRO_RENDER (ctx->render);
+    cairo_t *cr = render->cr;
     guint32 rgb = colour->rgb;
     double r, g, b;
 
@@ -282,22 +188,20 @@ static void
 _set_source_rsvg_pattern (RsvgDrawingCtx * ctx,
                           RsvgPattern * rsvg_pattern, guint8 opacity, RsvgBbox bbox)
 {
-    RsvgCairoRender *render = (RsvgCairoRender *) ctx->render;
+    RsvgCairoRender *render = RSVG_CAIRO_RENDER (ctx->render);
     RsvgPattern local_pattern = *rsvg_pattern;
     cairo_t *cr_render, *cr_pattern;
     cairo_pattern_t *pattern;
     cairo_surface_t *surface;
     cairo_matrix_t matrix;
-    int i;
-    double affine[6], caffine[6], bbwscale, bbhscale, scwscale, schscale;
-    double taffine[6], patternw, patternh, patternx, patterny;
+    cairo_matrix_t affine, caffine, taffine;
+    double bbwscale, bbhscale, scwscale, schscale;
+    double patternw, patternh, patternx, patterny;
     int pw, ph;
 
     rsvg_pattern = &local_pattern;
     rsvg_pattern_fix_fallback (rsvg_pattern);
     cr_render = render->cr;
-    _rsvg_affine_identity (affine);
-    _rsvg_affine_identity (caffine);
 
     if (rsvg_pattern->obj_bbox)
         _rsvg_push_view_box (ctx, 1., 1.);
@@ -315,17 +219,17 @@ _set_source_rsvg_pattern (RsvgDrawingCtx * ctx,
 
 
     if (rsvg_pattern->obj_bbox) {
-        bbwscale = bbox.w;
-        bbhscale = bbox.h;
+        bbwscale = bbox.rect.width;
+        bbhscale = bbox.rect.height;
     } else {
         bbwscale = 1.0;
         bbhscale = 1.0;
     }
 
-    _rsvg_affine_multiply (taffine, rsvg_pattern->affine, rsvg_current_state (ctx)->affine);
+    cairo_matrix_multiply (&taffine, &rsvg_pattern->affine, &rsvg_current_state (ctx)->affine);
 
-    scwscale = sqrt (taffine[0] * taffine[0] + taffine[2] * taffine[2]);
-    schscale = sqrt (taffine[1] * taffine[1] + taffine[3] * taffine[3]);
+    scwscale = sqrt (taffine.xx * taffine.xx + taffine.xy * taffine.xy);
+    schscale = sqrt (taffine.yx * taffine.yx + taffine.yy * taffine.yy);
 
     pw = patternw * bbwscale * scwscale;
     ph = patternh * bbhscale * schscale;
@@ -340,15 +244,15 @@ _set_source_rsvg_pattern (RsvgDrawingCtx * ctx,
     /* Create the pattern coordinate system */
     if (rsvg_pattern->obj_bbox) {
         /* subtract the pattern origin */
-        affine[4] = bbox.x + patternx * bbox.w;
-        affine[5] = bbox.y + patterny * bbox.h;
+        cairo_matrix_init_translate (&affine,
+                                     bbox.rect.x + patternx * bbox.rect.width,
+                                     bbox.rect.y + patterny * bbox.rect.height);
     } else {
         /* subtract the pattern origin */
-        affine[4] = patternx;
-        affine[5] = patterny;
+        cairo_matrix_init_translate (&affine, patternx, patterny);
     }
     /* Apply the pattern transform */
-    _rsvg_affine_multiply (affine, affine, rsvg_pattern->affine);
+    cairo_matrix_multiply (&affine, &affine, &rsvg_pattern->affine);
 
     /* Create the pattern contents coordinate system */
     if (rsvg_pattern->vbox.active) {
@@ -359,29 +263,35 @@ _set_source_rsvg_pattern (RsvgDrawingCtx * ctx,
         x = 0;
         y = 0;
         rsvg_preserve_aspect_ratio (rsvg_pattern->preserve_aspect_ratio,
-                                    rsvg_pattern->vbox.w, rsvg_pattern->vbox.h, &w, &h, &x, &y);
+                                    rsvg_pattern->vbox.rect.width, rsvg_pattern->vbox.rect.height,
+                                    &w, &h, &x, &y);
 
-        x -= rsvg_pattern->vbox.x * w / rsvg_pattern->vbox.w;
-        y -= rsvg_pattern->vbox.y * h / rsvg_pattern->vbox.h;
+        x -= rsvg_pattern->vbox.rect.x * w / rsvg_pattern->vbox.rect.width;
+        y -= rsvg_pattern->vbox.rect.y * h / rsvg_pattern->vbox.rect.height;
 
-        caffine[0] = w / rsvg_pattern->vbox.w;
-        caffine[3] = h / rsvg_pattern->vbox.h;
-        caffine[4] = x;
-        caffine[5] = y;
-        _rsvg_push_view_box (ctx, rsvg_pattern->vbox.w, rsvg_pattern->vbox.h);
+        cairo_matrix_init (&caffine,
+                           w / rsvg_pattern->vbox.rect.width,
+                           0,
+                           0,
+                           h / rsvg_pattern->vbox.rect.height,
+                           x,
+                           y);
+        _rsvg_push_view_box (ctx, rsvg_pattern->vbox.rect.width, rsvg_pattern->vbox.rect.height);
     } else if (rsvg_pattern->obj_cbbox) {
         /* If coords are in terms of the bounding box, use them */
-        caffine[0] = bbox.w;
-        caffine[3] = bbox.h;
+        cairo_matrix_init_scale (&caffine, bbox.rect.width, bbox.rect.height);
         _rsvg_push_view_box (ctx, 1., 1.);
+    } else {
+        cairo_matrix_init_identity (&caffine);
     }
 
     if (scwscale != 1.0 || schscale != 1.0) {
-        double scalematrix[6];
-        _rsvg_affine_scale (scalematrix, scwscale, schscale);
-        _rsvg_affine_multiply (caffine, caffine, scalematrix);
-        _rsvg_affine_scale (scalematrix, 1. / scwscale, 1. / schscale);
-        _rsvg_affine_multiply (affine, scalematrix, affine);
+        cairo_matrix_t scalematrix;
+
+        cairo_matrix_init_scale (&scalematrix, scwscale, schscale);
+        cairo_matrix_multiply (&caffine, &caffine, &scalematrix);
+        cairo_matrix_init_scale (&scalematrix, 1. / scwscale, 1. / schscale);
+        cairo_matrix_multiply (&affine, &scalematrix, &affine);
     }
 
     /* Draw to another surface */
@@ -389,9 +299,8 @@ _set_source_rsvg_pattern (RsvgDrawingCtx * ctx,
 
     /* Set up transformations to be determined by the contents units */
     rsvg_state_push (ctx);
-    for (i = 0; i < 6; i++)
-        rsvg_current_state (ctx)->personal_affine[i] =
-            rsvg_current_state (ctx)->affine[i] = caffine[i];
+    rsvg_current_state (ctx)->personal_affine =
+            rsvg_current_state (ctx)->affine = caffine;
 
     /* Draw everything */
     _rsvg_node_draw_children ((RsvgNode *) rsvg_pattern, ctx, 2);
@@ -404,10 +313,10 @@ _set_source_rsvg_pattern (RsvgDrawingCtx * ctx,
     pattern = cairo_pattern_create_for_surface (surface);
     cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
 
-    cairo_matrix_init (&matrix, affine[0], affine[1], affine[2], affine[3], affine[4], affine[5]);
+    matrix = affine;
+    if (cairo_matrix_invert (&matrix) != CAIRO_STATUS_SUCCESS)
+      goto out;
 
-
-    cairo_matrix_invert (&matrix);
     cairo_pattern_set_matrix (pattern, &matrix);
     cairo_pattern_set_filter (pattern, CAIRO_FILTER_BEST);
 
@@ -416,6 +325,8 @@ _set_source_rsvg_pattern (RsvgDrawingCtx * ctx,
     cairo_pattern_destroy (pattern);
     cairo_destroy (cr_pattern);
     cairo_surface_destroy (surface);
+
+  out:
     if (rsvg_pattern->obj_cbbox || rsvg_pattern->vbox.active)
         _rsvg_pop_view_box (ctx);
 }
@@ -444,17 +355,17 @@ _set_source_rsvg_paint_server (RsvgDrawingCtx * ctx,
 }
 
 static void
-_set_rsvg_affine (RsvgCairoRender * render, const double affine[6])
+_set_rsvg_affine (RsvgCairoRender * render, cairo_matrix_t *affine)
 {
     cairo_t * cr = render->cr;
     cairo_matrix_t matrix;
     gboolean nest = cr != render->initial_cr;
 
     cairo_matrix_init (&matrix,
-                       affine[0], affine[1],
-                       affine[2], affine[3],
-                       affine[4] + (nest ? 0 : render->offset_x),
-                       affine[5] + (nest ? 0 : render->offset_y));
+                       affine->xx, affine->yx,
+                       affine->xy, affine->yy,
+                       affine->x0 + (nest ? 0 : render->offset_x),
+                       affine->y0 + (nest ? 0 : render->offset_y));
     cairo_set_matrix (cr, &matrix);
 }
 
@@ -463,7 +374,7 @@ rsvg_cairo_create_pango_context (RsvgDrawingCtx * ctx)
 {
     PangoFontMap *fontmap;
     PangoContext *context;
-    RsvgCairoRender *render = (RsvgCairoRender *) ctx->render;
+    RsvgCairoRender *render = RSVG_CAIRO_RENDER (ctx->render);
 
     fontmap = pango_cairo_font_map_get_default ();
     context = pango_cairo_font_map_create_context (PANGO_CAIRO_FONT_MAP (fontmap));
@@ -475,22 +386,22 @@ rsvg_cairo_create_pango_context (RsvgDrawingCtx * ctx)
 void
 rsvg_cairo_render_pango_layout (RsvgDrawingCtx * ctx, PangoLayout * layout, double x, double y)
 {
-    RsvgCairoRender *render = (RsvgCairoRender *) ctx->render;
+    RsvgCairoRender *render = RSVG_CAIRO_RENDER (ctx->render);
     RsvgState *state = rsvg_current_state (ctx);
     PangoRectangle ink;
     RsvgBbox bbox;
 
-    _rsvg_cairo_set_text_antialias (render->cr, state->text_rendering_type);
+    cairo_set_antialias (render->cr, state->text_rendering_type);
 
-    _set_rsvg_affine (render, state->affine);
+    _set_rsvg_affine (render, &state->affine);
 
     pango_layout_get_extents (layout, &ink, NULL);
 
-    rsvg_bbox_init (&bbox, state->affine);
-    bbox.x = x + ink.x / (double)PANGO_SCALE;
-    bbox.y = y + ink.y / (double)PANGO_SCALE;
-    bbox.w = ink.width / (double)PANGO_SCALE;
-    bbox.h = ink.height / (double)PANGO_SCALE;
+    rsvg_bbox_init (&bbox, &state->affine);
+    bbox.rect.x = x + ink.x / (double)PANGO_SCALE;
+    bbox.rect.y = y + ink.y / (double)PANGO_SCALE;
+    bbox.rect.width = ink.width / (double)PANGO_SCALE;
+    bbox.rect.height = ink.height / (double)PANGO_SCALE;
     bbox.virgin = 0;
 
     if (state->fill) {
@@ -527,13 +438,11 @@ rsvg_cairo_render_pango_layout (RsvgDrawingCtx * ctx, PangoLayout * layout, doub
 }
 
 void
-rsvg_cairo_render_path (RsvgDrawingCtx * ctx, const RsvgBpathDef * bpath_def)
+rsvg_cairo_render_path (RsvgDrawingCtx * ctx, const cairo_path_t *path)
 {
-    RsvgCairoRender *render = (RsvgCairoRender *) ctx->render;
+    RsvgCairoRender *render = RSVG_CAIRO_RENDER (ctx->render);
     RsvgState *state = rsvg_current_state (ctx);
     cairo_t *cr;
-    RsvgBpath *bpath;
-    int i;
     int need_tmpbuf = 0;
     RsvgBbox bbox;
     double backup_tolerance;
@@ -543,16 +452,16 @@ rsvg_cairo_render_path (RsvgDrawingCtx * ctx, const RsvgBpathDef * bpath_def)
 
     need_tmpbuf = ((state->fill != NULL) && (state->stroke != NULL) && state->opacity != 0xff)
         || state->clip_path_ref || state->mask || state->filter
-        || (state->comp_op != RSVG_COMP_OP_SRC_OVER);
+        || (state->comp_op != CAIRO_OPERATOR_OVER);
 
     if (need_tmpbuf)
         rsvg_cairo_push_discrete_layer (ctx);
 
     cr = render->cr;
 
-	_rsvg_cairo_set_shape_antialias (cr, state->shape_rendering_type);
+    cairo_set_antialias (cr, state->shape_rendering_type);
 
-    _set_rsvg_affine (render, state->affine);
+    _set_rsvg_affine (render, &state->affine);
 
     cairo_set_line_width (cr, _rsvg_css_normalize_length (&state->stroke_width, ctx, 'h'));
     cairo_set_miter_limit (cr, state->miter_limit);
@@ -561,28 +470,9 @@ rsvg_cairo_render_path (RsvgDrawingCtx * ctx, const RsvgBpathDef * bpath_def)
     cairo_set_dash (cr, state->dash.dash, state->dash.n_dash,
                     _rsvg_css_normalize_length (&state->dash.offset, ctx, 'o'));
 
-    for (i = 0; i < bpath_def->n_bpath; i++) {
-        bpath = &bpath_def->bpath[i];
+    cairo_append_path (cr, path);
 
-        switch (bpath->code) {
-        case RSVG_MOVETO:
-            cairo_close_path (cr);
-            /* fall-through */
-        case RSVG_MOVETO_OPEN:
-            cairo_move_to (cr, bpath->x3, bpath->y3);
-            break;
-        case RSVG_CURVETO:
-            cairo_curve_to (cr, bpath->x1, bpath->y1, bpath->x2, bpath->y2, bpath->x3, bpath->y3);
-            break;
-        case RSVG_LINETO:
-            cairo_line_to (cr, bpath->x3, bpath->y3);
-            break;
-        case RSVG_END:
-            break;
-        }
-    }
-
-    rsvg_bbox_init (&bbox, state->affine);
+    rsvg_bbox_init (&bbox, &state->affine);
 
     backup_tolerance = cairo_get_tolerance (cr);
     cairo_set_tolerance (cr, 1.0);
@@ -592,19 +482,19 @@ rsvg_cairo_render_path (RsvgDrawingCtx * ctx, const RsvgBpathDef * bpath_def)
 
     if (state->fill != NULL) {
         RsvgBbox fb;
-        rsvg_bbox_init (&fb, state->affine);
-        cairo_fill_extents (cr, &fb.x, &fb.y, &fb.w, &fb.h);
-        fb.w -= fb.x;
-        fb.h -= fb.y;
+        rsvg_bbox_init (&fb, &state->affine);
+        cairo_fill_extents (cr, &fb.rect.x, &fb.rect.y, &fb.rect.width, &fb.rect.height);
+        fb.rect.width -= fb.rect.x;
+        fb.rect.height -= fb.rect.y;
         fb.virgin = 0;
         rsvg_bbox_insert (&bbox, &fb);
     }
     if (state->stroke != NULL) {
         RsvgBbox sb;
-        rsvg_bbox_init (&sb, state->affine);
-        cairo_stroke_extents (cr, &sb.x, &sb.y, &sb.w, &sb.h);
-        sb.w -= sb.x;
-        sb.h -= sb.y;
+        rsvg_bbox_init (&sb, &state->affine);
+        cairo_stroke_extents (cr, &sb.rect.x, &sb.rect.y, &sb.rect.width, &sb.rect.height);
+        sb.rect.width -= sb.rect.x;
+        sb.rect.height -= sb.rect.y;
         sb.virgin = 0;
         rsvg_bbox_insert (&bbox, &sb);
     }
@@ -615,10 +505,8 @@ rsvg_cairo_render_path (RsvgDrawingCtx * ctx, const RsvgBpathDef * bpath_def)
 
     if (state->fill != NULL) {
         int opacity;
-        if (state->fill_rule == FILL_RULE_EVENODD)
-            cairo_set_fill_rule (cr, CAIRO_FILL_RULE_EVEN_ODD);
-        else                    /* state->fill_rule == FILL_RULE_NONZERO */
-            cairo_set_fill_rule (cr, CAIRO_FILL_RULE_WINDING);
+
+        cairo_set_fill_rule (cr, state->fill_rule);
 
         if (!need_tmpbuf)
             opacity = (state->fill_opacity * state->opacity) / 255;
@@ -659,7 +547,7 @@ void
 rsvg_cairo_render_image (RsvgDrawingCtx * ctx, const GdkPixbuf * pixbuf,
                          double pixbuf_x, double pixbuf_y, double w, double h)
 {
-    RsvgCairoRender *render = (RsvgCairoRender *) ctx->render;
+    RsvgCairoRender *render = RSVG_CAIRO_RENDER (ctx->render);
     RsvgState *state = rsvg_current_state (ctx);
 
     gint width = gdk_pixbuf_get_width (pixbuf);
@@ -681,14 +569,14 @@ rsvg_cairo_render_image (RsvgDrawingCtx * ctx, const GdkPixbuf * pixbuf,
     if (!cairo_pixels)
         return;
 
-    rsvg_bbox_init (&bbox, state->affine);
-    bbox.x = pixbuf_x;
-    bbox.y = pixbuf_y;
-    bbox.w = w;
-    bbox.h = h;
+    rsvg_bbox_init (&bbox, &state->affine);
+    bbox.rect.x = pixbuf_x;
+    bbox.rect.y = pixbuf_y;
+    bbox.rect.width = w;
+    bbox.rect.height = h;
     bbox.virgin = 0;
 
-    _set_rsvg_affine (render, state->affine);
+    _set_rsvg_affine (render, &state->affine);
     cairo_scale (render->cr, w / dwidth, h / dheight);
     pixbuf_x *= dwidth / w;
     pixbuf_y *= dheight / h;
@@ -752,7 +640,7 @@ rsvg_cairo_render_image (RsvgDrawingCtx * ctx, const GdkPixbuf * pixbuf,
         cairo_pixels += 4 * width;
     }
 
-    _rsvg_cairo_set_operator (render->cr, state->comp_op);
+    cairo_set_operator (render->cr, state->comp_op);
 
 #if 1
     cairo_set_source_surface (render->cr, surface, pixbuf_x, pixbuf_y);
@@ -781,14 +669,14 @@ rsvg_cairo_render_image (RsvgDrawingCtx * ctx, const GdkPixbuf * pixbuf,
 static void
 rsvg_cairo_generate_mask (cairo_t * cr, RsvgMask * self, RsvgDrawingCtx * ctx, RsvgBbox * bbox)
 {
+    RsvgCairoRender *render = RSVG_CAIRO_RENDER (ctx->render);
     cairo_surface_t *surface;
     cairo_t *mask_cr, *save_cr;
-    RsvgCairoRender *render = (RsvgCairoRender *) ctx->render;
     RsvgState *state = rsvg_current_state (ctx);
     guint8 *pixels;
     guint32 width = render->width, height = render->height;
     guint32 rowstride = width * 4, row, i;
-    double affinesave[6];
+    cairo_matrix_t affinesave;
     double sx, sy, sw, sh;
     gboolean nest = cr != render->initial_cr;
 
@@ -817,23 +705,25 @@ rsvg_cairo_generate_mask (cairo_t * cr, RsvgMask * self, RsvgDrawingCtx * ctx, R
 
     if (self->maskunits == objectBoundingBox)
         rsvg_cairo_add_clipping_rect (ctx,
-                                      sx * bbox->w + bbox->x,
-                                      sy * bbox->h + bbox->y, sw * bbox->w, sh * bbox->h);
+                                      sx * bbox->rect.width + bbox->rect.x,
+                                      sy * bbox->rect.height + bbox->rect.y,
+                                      sw * bbox->rect.width,
+                                      sh * bbox->rect.height);
     else
         rsvg_cairo_add_clipping_rect (ctx, sx, sy, sw, sh);
 
     /* Horribly dirty hack to have the bbox premultiplied to everything */
     if (self->contentunits == objectBoundingBox) {
-        double bbtransform[6];
-        bbtransform[0] = bbox->w;
-        bbtransform[1] = 0.;
-        bbtransform[2] = 0.;
-        bbtransform[3] = bbox->h;
-        bbtransform[4] = bbox->x;
-        bbtransform[5] = bbox->y;
-        for (i = 0; i < 6; i++)
-            affinesave[i] = self->super.state->affine[i];
-        _rsvg_affine_multiply (self->super.state->affine, bbtransform, self->super.state->affine);
+        cairo_matrix_t bbtransform;
+        cairo_matrix_init (&bbtransform,
+                           bbox->rect.width,
+                           0,
+                           0,
+                           bbox->rect.height,
+                           bbox->rect.x,
+                           bbox->rect.y);
+        affinesave = self->super.state->affine;
+        cairo_matrix_multiply (&self->super.state->affine, &bbtransform, &self->super.state->affine);
         _rsvg_push_view_box (ctx, 1, 1);
     }
 
@@ -843,8 +733,7 @@ rsvg_cairo_generate_mask (cairo_t * cr, RsvgMask * self, RsvgDrawingCtx * ctx, R
 
     if (self->contentunits == objectBoundingBox) {
         _rsvg_pop_view_box (ctx);
-        for (i = 0; i < 6; i++)
-            self->super.state->affine[i] = affinesave[i];
+        self->super.state->affine = affinesave;
     }
 
     render->cr = save_cr;
@@ -871,7 +760,9 @@ rsvg_cairo_generate_mask (cairo_t * cr, RsvgMask * self, RsvgDrawingCtx * ctx, R
 static void
 rsvg_cairo_push_early_clips (RsvgDrawingCtx * ctx)
 {
-    cairo_save (((RsvgCairoRender *) ctx->render)->cr);
+    RsvgCairoRender *render = RSVG_CAIRO_RENDER (ctx->render);
+  
+    cairo_save (render->cr);
     if (rsvg_current_state (ctx)->clip_path_ref)
         if (((RsvgClipPath *) rsvg_current_state (ctx)->clip_path_ref)->units == userSpaceOnUse)
             rsvg_cairo_clip (ctx, rsvg_current_state (ctx)->clip_path_ref, NULL);
@@ -883,7 +774,7 @@ rsvg_cairo_push_render_stack (RsvgDrawingCtx * ctx)
 {
     /* XXX: Untested, probably needs help wrt filters */
 
-    RsvgCairoRender *render = (RsvgCairoRender *) ctx->render;
+    RsvgCairoRender *render = RSVG_CAIRO_RENDER (ctx->render);
     cairo_surface_t *surface;
     cairo_t *child_cr;
     RsvgBbox *bbox;
@@ -895,7 +786,7 @@ rsvg_cairo_push_render_stack (RsvgDrawingCtx * ctx)
             lateclip = TRUE;
 
     if (state->opacity == 0xFF
-        && !state->filter && !state->mask && !lateclip && (state->comp_op == RSVG_COMP_OP_SRC_OVER)
+        && !state->filter && !state->mask && !lateclip && (state->comp_op == CAIRO_OPERATOR_OVER)
         && (state->enable_background == RSVG_ENABLE_BACKGROUND_ACCUMULATE))
         return;
     if (!state->filter)
@@ -940,7 +831,7 @@ rsvg_cairo_push_render_stack (RsvgDrawingCtx * ctx)
     bbox = g_new (RsvgBbox, 1);
     *bbox = render->bbox;
     render->bb_stack = g_list_prepend (render->bb_stack, bbox);
-    rsvg_bbox_init (&render->bbox, state->affine);
+    rsvg_bbox_init (&render->bbox, &state->affine);
 }
 
 void
@@ -953,7 +844,7 @@ rsvg_cairo_push_discrete_layer (RsvgDrawingCtx * ctx)
 static void
 rsvg_cairo_pop_render_stack (RsvgDrawingCtx * ctx)
 {
-    RsvgCairoRender *render = (RsvgCairoRender *) ctx->render;
+    RsvgCairoRender *render = RSVG_CAIRO_RENDER (ctx->render);
     cairo_t *child_cr = render->cr;
     gboolean lateclip = FALSE;
     cairo_surface_t *surface = NULL;
@@ -965,7 +856,7 @@ rsvg_cairo_pop_render_stack (RsvgDrawingCtx * ctx)
             lateclip = TRUE;
 
     if (state->opacity == 0xFF
-        && !state->filter && !state->mask && !lateclip && (state->comp_op == RSVG_COMP_OP_SRC_OVER)
+        && !state->filter && !state->mask && !lateclip && (state->comp_op == CAIRO_OPERATOR_OVER)
         && (state->enable_background == RSVG_ENABLE_BACKGROUND_ACCUMULATE))
         return;
 
@@ -1003,7 +894,7 @@ rsvg_cairo_pop_render_stack (RsvgDrawingCtx * ctx)
     if (lateclip)
         rsvg_cairo_clip (ctx, rsvg_current_state (ctx)->clip_path_ref, &render->bbox);
 
-    _rsvg_cairo_set_operator (render->cr, state->comp_op);
+    cairo_set_operator (render->cr, state->comp_op);
 
     if (state->mask) {
         rsvg_cairo_generate_mask (render->cr, state->mask, ctx, &render->bbox);
@@ -1029,17 +920,19 @@ rsvg_cairo_pop_render_stack (RsvgDrawingCtx * ctx)
 void
 rsvg_cairo_pop_discrete_layer (RsvgDrawingCtx * ctx)
 {
+    RsvgCairoRender *render = RSVG_CAIRO_RENDER (ctx->render);
+
     rsvg_cairo_pop_render_stack (ctx);
-    cairo_restore (((RsvgCairoRender *) ctx->render)->cr);
+    cairo_restore (render->cr);
 }
 
 void
 rsvg_cairo_add_clipping_rect (RsvgDrawingCtx * ctx, double x, double y, double w, double h)
 {
-    RsvgCairoRender *render = (RsvgCairoRender *) ctx->render;
+    RsvgCairoRender *render = RSVG_CAIRO_RENDER (ctx->render);
     cairo_t *cr = render->cr;
 
-    _set_rsvg_affine (render, rsvg_current_state (ctx)->affine);
+    _set_rsvg_affine (render, &rsvg_current_state (ctx)->affine);
 
     cairo_rectangle (cr, x, y, w, h);
     cairo_clip (cr);
