@@ -181,12 +181,13 @@ main (int argc, char **argv)
     /* Set the locale so that UTF-8 filenames work */
     setlocale(LC_ALL, "");
 
-    g_thread_init(NULL);
+    g_type_init ();
 
     g_option_context = g_option_context_new (_("- SVG Converter"));
     g_option_context_add_main_entries (g_option_context, options_table, NULL);
     g_option_context_set_help_enabled (g_option_context, TRUE);
     if (!g_option_context_parse (g_option_context, &argc, &argv, &error)) {
+        g_option_context_free (g_option_context);
         display_error (error);
         exit (1);
     }
@@ -202,8 +203,11 @@ main (int argc, char **argv)
         output_file = fopen (output, "wb");
         if (!output_file) {
             fprintf (stderr, _("Error saving to file: %s\n"), output);
+            g_free (output);
             exit (1);
         }
+
+        g_free (output);
     }
 
     if (args)
@@ -221,7 +225,6 @@ main (int argc, char **argv)
     if (zoom != 1.0)
         x_zoom = y_zoom = zoom;
 
-    rsvg_init ();
     rsvg_set_default_dpi_x_y (dpi_x, dpi_y);
 
     for (i = 0; i < n_args; i++) {
@@ -359,7 +362,9 @@ main (int argc, char **argv)
 
     fclose (output_file);
 
-    rsvg_term ();
+    g_strfreev (args);
+
+    rsvg_cleanup ();
 
     return 0;
 }
