@@ -21,11 +21,29 @@
    Boston, MA 02111-1307, USA.
 */
 
+/**
+ * SECTION: rsvg-handle
+ * @short_description: Create and manipulate SVG objects
+ *
+ * librsvg is a component used within software applications to enable
+ * support for SVG-format scalable graphics. In contrast to raster
+ * formats, scalable vector graphics provide users and artists a way
+ * to create, view, and provide imagery that is not limited to the
+ * pixel or dot density that an output device is capable of.
+ *
+ * Many software developers use the librsvg library to render
+ * SVG graphics. It is lightweight and portable.
+ */
+
+#include "config.h"
+
 #include "rsvg-private.h"
 #include "rsvg-defs.h"
+#include "librsvg-enum-types.h"
 
 enum {
     PROP_0,
+    PROP_FLAGS,
     PROP_DPI_X,
     PROP_DPI_Y,
     PROP_BASE_URI,
@@ -47,10 +65,15 @@ static GObjectClass *rsvg_parent_class = NULL;
 static void
 instance_init (RsvgHandle * self)
 {
-    self->priv = g_new0 (RsvgHandlePrivate, 1);
+    self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, RSVG_TYPE_HANDLE, RsvgHandlePrivate);
+
+    self->priv->flags = RSVG_HANDLE_FLAGS_NONE;
     self->priv->defs = rsvg_defs_new ();
     self->priv->handler_nest = 0;
-    self->priv->entities = g_hash_table_new (g_str_hash, g_str_equal);
+    self->priv->entities = g_hash_table_new_full (g_str_hash, 
+                                                  g_str_equal,
+                                                  g_free,
+                                                  (GDestroyNotify) xmlFreeNode);
     self->priv->dpi_x = rsvg_internal_dpi_x;
     self->priv->dpi_y = rsvg_internal_dpi_y;
 
@@ -64,44 +87,23 @@ instance_init (RsvgHandle * self)
     self->priv->treebase = NULL;
 
     self->priv->finished = 0;
+    self->priv->data_input_stream = NULL;
     self->priv->first_write = TRUE;
 
     self->priv->is_disposed = FALSE;
     self->priv->in_loop = FALSE;
 }
 
-static void
-rsvg_ctx_free_helper (gpointer key, gpointer value, gpointer user_data)
-{
-    xmlEntityPtr entval = (xmlEntityPtr) value;
-
-#if LIBXML_VERSION < 20700
-    /* key == entval->name, so it's implicitly freed below */
-
-    xmlFree ((xmlChar *) entval->name);
-    xmlFree ((xmlChar *) entval->ExternalID);
-    xmlFree ((xmlChar *) entval->SystemID);
-    xmlFree (entval->content);
-    xmlFree (entval->orig);
-    xmlFree (entval);
-#else
-    xmlFreeNode((xmlNode *) entval);
-#endif
-}
-
 static void
 instance_dispose (GObject * instance)
 {
     RsvgHandle *self = (RsvgHandle *) instance;
 
+    if (self->priv->is_disposed)
+      goto chain;
+
     self->priv->is_disposed = TRUE;
 
-#if HAVE_SVGZ
-    if (self->priv->is_gzipped)
-        g_object_unref (self->priv->gzipped_data);
-#endif
-
-    g_hash_table_foreach (self->priv->entities, rsvg_ctx_free_helper, NULL);
     g_hash_table_destroy (self->priv->entities);
     rsvg_defs_free (self->priv->defs);
     g_hash_table_destroy (self->priv->css_props);
@@ -118,8 +120,16 @@ instance_dispose (GObject * instance)
     if (self->priv->base_uri)
         g_free (self->priv->base_uri);
 
-    g_free (self->priv);
+    if (self->priv->base_gfile) {
+        g_object_unref (self->priv->base_gfile);
+        self->priv->base_gfile = NULL;
+    }
+    if (self->priv->data_input_stream) {
+        g_object_unref (self->priv->data_input_stream);
+        self->priv->data_input_stream = NULL;
+    }
 
+  chain:
     rsvg_parent_class->dispose (instance);
 }
 
@@ -129,6 +139,9 @@ set_property (GObject * instance, guint prop_id, GValue const *value, GParamSpec
     RsvgHandle *self = RSVG_HANDLE (instance);
 
     switch (prop_id) {
+    case PROP_FLAGS:
+        self->priv->flags = g_value_get_flags (value);
+        break;
     case PROP_DPI_X:
         rsvg_handle_set_dpi_x_y (self, g_value_get_double (value), self->priv->dpi_y);
         break;
@@ -150,6 +163,9 @@ get_property (GObject * instance, guint prop_id, GValue * value, GParamSpec * ps
     RsvgDimensionData dim;
 
     switch (prop_id) {
+    case PROP_FLAGS:
+        g_value_set_flags (value, self->priv->flags);
+        break;
     case PROP_DPI_X:
         g_value_set_double (value, self->priv->dpi_x);
         break;
@@ -202,6 +218,20 @@ class_init (RsvgHandleClass * klass)
     gobject_class->set_property = set_property;
     gobject_class->get_property = get_property;
 
+    /**
+     * RsvgHandle:flags:
+     * 
+     * Flags from #RsvgHandleFlags.
+     * 
+     * Since: 2.36
+     */
+    g_object_class_install_property (gobject_class,
+                                     PROP_FLAGS,
+                                     g_param_spec_flags ("flags", NULL, NULL,
+                                                         RSVG_TYPE_HANDLE_FLAGS,
+                                                         RSVG_HANDLE_FLAGS_NONE,
+                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
     /**
      * dpi-x:
      */
@@ -270,6 +300,10 @@ class_init (RsvgHandleClass * klass)
                                                           _("SVG file metadata"), NULL,
                                                           (GParamFlags) (G_PARAM_READABLE)));
 
+    g_type_class_add_private (klass, sizeof (RsvgHandlePrivate));
+
+    xmlInitParser ();
+
     rsvg_SAX_handler_struct_init ();
 }
 
@@ -337,3 +371,22 @@ rsvg_handle_new (void)
 {
     return RSVG_HANDLE (g_object_new (RSVG_TYPE_HANDLE, NULL));
 }
+
+
+/**
+ * rsvg_handle_new_with_flags:
+ * @flags: flags from #RsvgHandleFlags
+ *
+ * Creates a new #RsvgHandle with flags @flags.
+ *
+ * Returns: (transfer full): a new #RsvgHandle
+ * 
+ * Since: 2.36
+ **/
+RsvgHandle *
+rsvg_handle_new_with_flags (RsvgHandleFlags flags)
+{
+    return g_object_new (RSVG_TYPE_HANDLE, 
+                         "flags", flags,
+                         NULL);
+}
