@@ -27,8 +27,9 @@
 #ifndef RSVG_PRIVATE_H
 #define RSVG_PRIVATE_H
 
+#include <cairo.h>
+
 #include "rsvg.h"
-#include "rsvg-bpath-util.h"
 
 #include <libxml/SAX.h>
 #include <libxml/xmlmemory.h>
@@ -125,6 +126,8 @@ struct RsvgSaxHandler {
 };
 
 struct RsvgHandlePrivate {
+    RsvgHandleFlags flags;
+
     gboolean is_disposed;
     gboolean is_closed;
 
@@ -169,16 +172,12 @@ struct RsvgHandlePrivate {
     gboolean in_loop;		/* see get_dimension() */
 
     gboolean first_write;
-#if GLIB_CHECK_VERSION (2, 24, 0)
     GInputStream *data_input_stream; /* for rsvg_handle_write of svgz data */
-#elif defined(HAVE_GSF)
-    void *gzipped_data;         /* really a GsfOutput */
-#endif
 };
 
 typedef struct {
+    cairo_rectangle_t rect;
     gboolean active;
-    double x, y, w, h;
 } RsvgViewBox;
 
 /*Contextual information for the drawing phase*/
@@ -199,13 +198,24 @@ struct RsvgDrawingCtx {
 
 /*Abstract base class for context for our backends (one as yet)*/
 
+typedef enum {
+  RSVG_RENDER_TYPE_INVALID,
+
+  RSVG_RENDER_TYPE_BASE,
+
+  RSVG_RENDER_TYPE_CAIRO = 8,
+  RSVG_RENDER_TYPE_CAIRO_CLIP
+} RsvgRenderType;
+
 struct RsvgRender {
+    RsvgRenderType type;
+
     void (*free) (RsvgRender * self);
 
     PangoContext    *(*create_pango_context)    (RsvgDrawingCtx * ctx);
     void             (*render_pango_layout)	    (RsvgDrawingCtx * ctx, PangoLayout *layout,
                                                  double x, double y);
-    void             (*render_path)             (RsvgDrawingCtx * ctx, const RsvgBpathDef * path);
+    void             (*render_path)             (RsvgDrawingCtx * ctx, const cairo_path_t *path);
     void             (*render_image)            (RsvgDrawingCtx * ctx, const GdkPixbuf * pixbuf,
                                                  double x, double y, double w, double h);
     void             (*pop_discrete_layer)      (RsvgDrawingCtx * ctx);
@@ -216,6 +226,16 @@ struct RsvgRender {
                                                  double w, double h);
 };
 
+static inline RsvgRender *
+_rsvg_render_check_type (RsvgRender *render,
+                         RsvgRenderType type)
+{
+  g_assert ((render->type & type) == type);
+  return render;
+}
+
+#define _RSVG_RENDER_CIC(render, render_type, RenderCType) \
+  ((RenderCType*) _rsvg_render_check_type ((render), (render_type)))
 
 typedef struct {
     double length;
@@ -227,9 +247,9 @@ struct _RsvgIRect {
 };
 
 typedef struct {
-    gdouble x, y, w, h;
+    cairo_rectangle_t rect;
+    cairo_matrix_t affine;
     gboolean virgin;
-    double affine[6];
 } RsvgBbox;
 
 typedef enum {
@@ -331,7 +351,7 @@ struct _RsvgNodeChars {
 typedef void (*RsvgPropertyBagEnumFunc) (const char *key, const char *value, gpointer user_data);
 
 RsvgPropertyBag	    *rsvg_property_bag_new       (const char **atts);
-RsvgPropertyBag	    *rsvg_property_bag_ref       (RsvgPropertyBag * bag);
+RsvgPropertyBag	    *rsvg_property_bag_dup       (RsvgPropertyBag * bag);
 void                 rsvg_property_bag_free      (RsvgPropertyBag * bag);
 const char          *rsvg_property_bag_lookup    (RsvgPropertyBag * bag, const char *key);
 guint                rsvg_property_bag_size	     (RsvgPropertyBag * bag);
@@ -353,7 +373,7 @@ GByteArray  *_rsvg_acquire_xlink_href_resource  (const char *href,
 
 void rsvg_pop_discrete_layer    (RsvgDrawingCtx * ctx);
 void rsvg_push_discrete_layer   (RsvgDrawingCtx * ctx);
-void rsvg_render_path           (RsvgDrawingCtx * ctx, const char *d);
+void rsvg_render_path           (RsvgDrawingCtx * ctx, const cairo_path_t *path);
 void rsvg_render_image          (RsvgDrawingCtx * ctx, GdkPixbuf * pb,
                                  double x, double y, double w, double h);
 void rsvg_render_free           (RsvgRender * render);
@@ -361,46 +381,11 @@ void rsvg_add_clipping_rect     (RsvgDrawingCtx * ctx, double x, double y, doubl
 GdkPixbuf *rsvg_get_image_of_node (RsvgDrawingCtx * ctx, RsvgNode * drawable, double w, double h);
 
 
-void _rsvg_affine_invert (double dst_affine[6], const double src_affine[6]);
-
-/* flip the matrix, FALSE, FALSE is a simple copy operation, and
-   TRUE, TRUE equals a rotation by 180 degrees */
-void _rsvg_affine_flip (double dst_affine[6], const double src_affine[6], int horz, int vert);
-
-void _rsvg_affine_multiply (double dst[6], const double src1[6], const double src2[6]);
-
-/* set up the identity matrix */
-void _rsvg_affine_identity (double dst[6]);
-
-/* set up a scaling matrix */
-void _rsvg_affine_scale (double dst[6], double sx, double sy);
-
-/* set up a rotation matrix; theta is given in degrees */
-void _rsvg_affine_rotate (double dst[6], double theta);
-
-/* set up a shearing matrix; theta is given in degrees */
-void _rsvg_affine_shear (double dst[6], double theta);
-
-/* set up a translation matrix */
-void _rsvg_affine_translate (double dst[6], double tx, double ty);
-
-
-/* find the affine's "expansion factor", i.e. the scale amount */
-double _rsvg_affine_expansion (const double src[6]);
-
-/* Determine whether the affine transformation is rectilinear,
-   i.e. whether a rectangle aligned to the grid is transformed into
-   another rectangle aligned to the grid. */
-int _rsvg_affine_rectilinear (const double src[6]);
-
-/* Determine whether two affine transformations are equal within grid allignment */
-int _rsvg_affine_equal (double matrix1[6], double matrix2[6]);
-
 void rsvg_node_set_atts (RsvgNode * node, RsvgHandle * ctx, RsvgPropertyBag * atts);
 
 void rsvg_drawing_ctx_free (RsvgDrawingCtx * handle);
 
-void rsvg_bbox_init     (RsvgBbox * self, double *affine);
+void rsvg_bbox_init     (RsvgBbox * self, cairo_matrix_t *matrix);
 void rsvg_bbox_insert   (RsvgBbox * dst, RsvgBbox * src);
 void rsvg_bbox_clip     (RsvgBbox * dst, RsvgBbox * src);
 
