@@ -27,6 +27,7 @@
 #ifndef RSVG_STYLES_H
 #define RSVG_STYLES_H
 
+#include <cairo.h>
 #include "rsvg.h"
 #include "rsvg-paint-server.h"
 
@@ -50,58 +51,12 @@ typedef enum {
     TEXT_ANCHOR_END
 } TextAnchor;
 
-enum {
-    FILL_RULE_EVENODD = 0,
-    FILL_RULE_NONZERO = 1
-};
-
 typedef enum {
     UNICODE_BIDI_NORMAL = 0,
     UNICODE_BIDI_EMBED = 1,
     UNICODE_BIDI_OVERRIDE = 2
 } UnicodeBidi;
 
-typedef enum {
-    SHAPE_RENDERING_AUTO = 0,
-    SHAPE_RENDERING_OPTIMIZE_SPEED,
-    SHAPE_RENDERING_CRISP_EDGES,
-    SHAPE_RENDERING_GEOMETRIC_PRECISION
-} ShapeRenderingProperty;
-
-typedef enum {
-    TEXT_RENDERING_AUTO = 0,
-    TEXT_RENDERING_OPTIMIZE_SPEED,
-    TEXT_RENDERING_OPTIMIZE_LEGIBILITY,
-    TEXT_RENDERING_GEOMETRIC_PRECISION
-} TextRenderingProperty;
-
-typedef enum {
-    RSVG_COMP_OP_CLEAR,
-    RSVG_COMP_OP_SRC,
-    RSVG_COMP_OP_DST,
-    RSVG_COMP_OP_SRC_OVER,
-    RSVG_COMP_OP_DST_OVER,
-    RSVG_COMP_OP_SRC_IN,
-    RSVG_COMP_OP_DST_IN,
-    RSVG_COMP_OP_SRC_OUT,
-    RSVG_COMP_OP_DST_OUT,
-    RSVG_COMP_OP_SRC_ATOP,
-    RSVG_COMP_OP_DST_ATOP,
-    RSVG_COMP_OP_XOR,
-    RSVG_COMP_OP_PLUS,
-    RSVG_COMP_OP_MULTIPLY,
-    RSVG_COMP_OP_SCREEN,
-    RSVG_COMP_OP_OVERLAY,
-    RSVG_COMP_OP_DARKEN,
-    RSVG_COMP_OP_LIGHTEN,
-    RSVG_COMP_OP_COLOR_DODGE,
-    RSVG_COMP_OP_COLOR_BURN,
-    RSVG_COMP_OP_HARD_LIGHT,
-    RSVG_COMP_OP_SOFT_LIGHT,
-    RSVG_COMP_OP_DIFFERENCE,
-    RSVG_COMP_OP_EXCLUSION
-} RsvgCompOpType;
-
 typedef enum {
     RSVG_ENABLE_BACKGROUND_ACCUMULATE,
     RSVG_ENABLE_BACKGROUND_NEW
@@ -109,18 +64,6 @@ typedef enum {
 
 /* enums and data structures are ABI compatible with libart */
 
-typedef enum {
-    RSVG_PATH_STROKE_JOIN_MITER,
-    RSVG_PATH_STROKE_JOIN_ROUND,
-    RSVG_PATH_STROKE_JOIN_BEVEL
-} RsvgPathStrokeJoinType;
-
-typedef enum {
-    RSVG_PATH_STROKE_CAP_BUTT,
-    RSVG_PATH_STROKE_CAP_ROUND,
-    RSVG_PATH_STROKE_CAP_SQUARE
-} RsvgPathStrokeCapType;
-
 typedef struct _RsvgVpathDash RsvgVpathDash;
 
 struct _RsvgVpathDash {
@@ -133,8 +76,8 @@ struct _RsvgVpathDash {
 
 struct _RsvgState {
     RsvgState *parent;
-    double affine[6];
-    double personal_affine[6];
+    cairo_matrix_t affine;
+    cairo_matrix_t personal_affine;
 
     RsvgFilter *filter;
     void *mask;
@@ -163,9 +106,9 @@ struct _RsvgState {
     double miter_limit;
     gboolean has_miter_limit;
 
-    RsvgPathStrokeCapType cap;
+    cairo_line_cap_t cap;
     gboolean has_cap;
-    RsvgPathStrokeJoinType join;
+    cairo_line_join_t join;
     gboolean has_join;
 
     RsvgLength font_size;
@@ -229,13 +172,13 @@ struct _RsvgState {
     gboolean has_middleMarker;
     gboolean has_endMarker;
 
-    RsvgCompOpType comp_op;
+    cairo_operator_t comp_op;
     RsvgEnableBackgroundType enable_background;
 
-    ShapeRenderingProperty shape_rendering_type;
+    cairo_antialias_t shape_rendering_type;
     gboolean has_shape_rendering_type;
-    
-    TextRenderingProperty text_rendering_type;
+
+    cairo_antialias_t text_rendering_type;
     gboolean has_text_rendering_type;
 
     GHashTable *styles;
@@ -263,7 +206,7 @@ void rsvg_parse_style_attrs (RsvgHandle * ctx, RsvgState * state, const char *ta
 gdouble rsvg_viewport_percentage (gdouble width, gdouble height);
 gdouble rsvg_dpi_percentage      (RsvgHandle * ctx);
 
-gboolean rsvg_parse_transform   (double dst[6], const char *src);
+gboolean rsvg_parse_transform   (cairo_matrix_t *matrix, const char *src);
 
 RsvgState *rsvg_state_parent    (RsvgState * state);
 
