@@ -26,10 +26,21 @@
 #ifndef RSVG_H
 #define RSVG_H
 
+#include <glib-object.h>
+#include <gio/gio.h>
+
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
 G_BEGIN_DECLS
 
+#if defined(RSVG_DISABLE_DEPRECATION_WARNINGS) || !GLIB_CHECK_VERSION (2, 31, 0)
+#define RSVG_DEPRECATED
+#define RSVG_DEPRECATED_FOR(f)
+#else
+#define RSVG_DEPRECATED G_DEPRECATED
+#define RSVG_DEPRECATED_FOR(f) G_DEPRECATED_FOR(f)
+#endif
+
 #define RSVG_TYPE_HANDLE                  (rsvg_handle_get_type ())
 #define RSVG_HANDLE(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), RSVG_TYPE_HANDLE, RsvgHandle))
 #define RSVG_HANDLE_CLASS(klass)          (G_TYPE_CHECK_CLASS_CAST ((klass), RSVG_TYPE_HANDLE, RsvgHandleClass))
@@ -107,8 +118,7 @@ struct _RsvgPositionData {
     int y;
 };
 
-void rsvg_init (void);
-void rsvg_term (void);
+void rsvg_cleanup (void);
 
 void rsvg_set_default_dpi	(double dpi);
 void rsvg_set_default_dpi_x_y	(double dpi_x, double dpi_y);
@@ -123,8 +133,8 @@ gboolean     rsvg_handle_close		(RsvgHandle * handle, GError ** error);
 GdkPixbuf   *rsvg_handle_get_pixbuf	(RsvgHandle * handle);
 GdkPixbuf   *rsvg_handle_get_pixbuf_sub (RsvgHandle * handle, const char *id);
 
-G_CONST_RETURN char	*rsvg_handle_get_base_uri (RsvgHandle * handle);
-void                 rsvg_handle_set_base_uri (RsvgHandle * handle, const char *base_uri);
+const char  *rsvg_handle_get_base_uri (RsvgHandle * handle);
+void         rsvg_handle_set_base_uri (RsvgHandle * handle, const char *base_uri);
 
 void rsvg_handle_get_dimensions (RsvgHandle * handle, RsvgDimensionData * dimension_data);
 
@@ -133,17 +143,51 @@ gboolean rsvg_handle_get_position_sub (RsvgHandle * handle, RsvgPositionData * p
 
 gboolean rsvg_handle_has_sub (RsvgHandle * handle, const char *id);
 
+/* GIO APIs */
+
+typedef enum /*< flags >*/ 
+{
+    RSVG_HANDLE_FLAGS_NONE        = 0
+} RsvgHandleFlags;
+
+RsvgHandle *rsvg_handle_new_with_flags (RsvgHandleFlags flags);
+
+void        rsvg_handle_set_base_gfile (RsvgHandle *handle,
+                                        GFile      *base_file);
+
+gboolean    rsvg_handle_read_stream_sync (RsvgHandle   *handle,
+                                          GInputStream *stream,
+                                          GCancellable *cancellable,
+                                          GError      **error);
+
+RsvgHandle *rsvg_handle_new_from_gfile_sync (GFile          *file,
+                                             RsvgHandleFlags flags,
+                                             GCancellable   *cancellable,
+                                             GError        **error);
+
+RsvgHandle *rsvg_handle_new_from_stream_sync (GInputStream   *input_stream,
+                                              GFile          *base_file,
+                                              RsvgHandleFlags flags,
+                                              GCancellable   *cancellable,
+                                              GError        **error);
+
 /* Accessibility API */
 
-G_CONST_RETURN char *rsvg_handle_get_title	(RsvgHandle * handle);
-G_CONST_RETURN char *rsvg_handle_get_desc	(RsvgHandle * handle);
-G_CONST_RETURN char *rsvg_handle_get_metadata	(RsvgHandle * handle);
+const char *rsvg_handle_get_title	(RsvgHandle * handle);
+const char *rsvg_handle_get_desc	(RsvgHandle * handle);
+const char *rsvg_handle_get_metadata	(RsvgHandle * handle);
 
 RsvgHandle *rsvg_handle_new_from_data (const guint8 * data, gsize data_len, GError ** error);
 RsvgHandle *rsvg_handle_new_from_file (const gchar * file_name, GError ** error);
 
-#ifndef RSVG_DISABLE_DEPRECATED
+/* BEGIN deprecated APIs. Do not use! */
 
+RSVG_DEPRECATED_FOR(g_type_init)
+void rsvg_init (void);
+RSVG_DEPRECATED
+void rsvg_term (void);
+
+RSVG_DEPRECATED_FOR(g_object_unref)
 void rsvg_handle_free (RsvgHandle * handle);
 
 /**
@@ -159,26 +203,33 @@ void rsvg_handle_free (RsvgHandle * handle);
  *
  * Deprecated: Set up a cairo matrix and use rsvg_handle_render_cairo() instead.
  */
-typedef void (*RsvgSizeFunc) (gint * width, gint * height, gpointer user_data);
+typedef RSVG_DEPRECATED void (*RsvgSizeFunc) (gint * width, gint * height, gpointer user_data);
+
+RSVG_DEPRECATED
 void rsvg_handle_set_size_callback (RsvgHandle * handle,
                                     RsvgSizeFunc size_func,
                                     gpointer user_data, GDestroyNotify user_data_destroy);
 
 /* GdkPixbuf convenience API */
 
+RSVG_DEPRECATED
 GdkPixbuf *rsvg_pixbuf_from_file            (const gchar * file_name, GError ** error);
+RSVG_DEPRECATED
 GdkPixbuf *rsvg_pixbuf_from_file_at_zoom    (const gchar * file_name,
                                              double x_zoom, double y_zoom, GError ** error);
+RSVG_DEPRECATED
 GdkPixbuf *rsvg_pixbuf_from_file_at_size    (const gchar * file_name, gint width, gint height,
                                              GError ** error);
+RSVG_DEPRECATED
 GdkPixbuf *rsvg_pixbuf_from_file_at_max_size    (const gchar * file_name,
                                                  gint max_width, gint max_height, GError ** error);
+RSVG_DEPRECATED
 GdkPixbuf *rsvg_pixbuf_from_file_at_zoom_with_max (const gchar * file_name,
                                                    double x_zoom,
                                                    double y_zoom,
                                                    gint max_width, gint max_height, GError ** error);
 
-#endif                          /* RSVG_DISABLE_DEPRECATED */
+/* END deprecated APIs. */
 
 G_END_DECLS
 
