@@ -72,7 +72,6 @@ get_image_size_from_data (ViewerCbInfo *info,
 {
     RsvgHandle *handle;
     RsvgDimensionData dimensions;
-    gboolean ret;
 
     handle = rsvg_handle_new ();
 
@@ -93,7 +92,7 @@ get_image_size_from_data (ViewerCbInfo *info,
         return FALSE;
     }
 
-    ret = rsvg_handle_get_dimensions_sub (handle, &dimensions, info->id);
+    (void) rsvg_handle_get_dimensions_sub (handle, &dimensions, info->id);
     g_object_unref (handle);
 
     if (width)
@@ -344,7 +343,6 @@ static void
 print_pixbuf (GObject * ignored, gpointer user_data)
 {
   GtkPrintOperation *print;
-  GtkPrintOperationResult res;
   ViewerCbInfo *info = (ViewerCbInfo *) user_data;
 
   print = gtk_print_operation_new ();
@@ -352,7 +350,7 @@ print_pixbuf (GObject * ignored, gpointer user_data)
   g_signal_connect (print, "begin_print", G_CALLBACK (begin_print), info);
   g_signal_connect (print, "draw_page", G_CALLBACK (draw_page), info);
 
-  res = gtk_print_operation_run (print, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
+  (void)gtk_print_operation_run (print, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
                                  GTK_WINDOW (info->window), NULL);
 
   g_object_unref (print);
@@ -613,7 +611,11 @@ populate_window (GtkWidget * win, ViewerCbInfo * info, int xid, gint win_width,
     GtkWidget *scroll;
     gint img_width, img_height;
 
+#if GTK_CHECK_VERSION (3, 2, 0)
+    vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+#else
     vbox = gtk_vbox_new (FALSE, 0);
+#endif
     gtk_container_add (GTK_CONTAINER (win), vbox);
 
     /* create a new image */
@@ -718,6 +720,7 @@ view_pixbuf (ViewerCbInfo * info, int xid, const char *color)
 int
 main (int argc, char **argv)
 {
+    int retval = 1;
     GError *err = NULL;
     GOptionContext *g_option_context;
     double x_zoom = 1.0;
@@ -770,7 +773,7 @@ main (int argc, char **argv)
 	/* Set the locale so that UTF-8 filenames work */
     setlocale(LC_ALL, "");
 
-    g_thread_init (NULL);
+    g_type_init ();
 
     info.pixbuf = NULL;
     info.svg_bytes = NULL;
@@ -802,9 +805,6 @@ main (int argc, char **argv)
         return 1;
     }
 
-    /* initialize gtk+ and rsvg */
-    rsvg_init ();
-
     rsvg_set_default_dpi_x_y (dpi_x, dpi_y);
 
     /* if both are unspecified, assume user wants to zoom the pixbuf in at least 1 dimension */
@@ -895,9 +895,11 @@ main (int argc, char **argv)
 
         g_print ("\n");
 
-        return 1;
+        goto done;
     }
 
+    retval = 0;
+
     info.accel_group = gtk_accel_group_new ();
 
     view_pixbuf (&info, xid, bg_color);
@@ -906,8 +908,12 @@ main (int argc, char **argv)
     gtk_main ();
 
     g_object_unref (info.pixbuf);
-    g_byte_array_free (info.svg_bytes, TRUE);
-    rsvg_term ();
 
-    return 0;
+  done:
+    g_byte_array_free (info.svg_bytes, TRUE);
+    g_strfreev (args);
+
+    rsvg_cleanup ();
+
+    return retval;
 }
